// Copyright 2001 year by Columbia University. All rights reserved.

// AudioDeviceTest.cpp : Defines the entry point for the console application.
// A test driver for the audio device layer.

#ifdef WIN32    
#pragma warning (disable:4786)
#endif

#ifdef WIN32
#include <windows.h>
#include <AudioDevices/Win32AudioDevice.h>
#define sleep(x) Sleep(x)
#endif

#ifdef LINUX
#include <unistd.h>
#endif

#include <AudioDevices/DeviceCaps.h>
#include <AudioDevices/AudioDevice.h>
#include <AudioDevices/AudioDeviceFactory.h>
#include <Utils/Debug.h>


int main(int argc, char* argv[])
{
  // The debug can be turned on and off.
  Debug::specify("D:M:T:O:debug.out");
  Debug D ("main", false);

  // Building the devidce returns the correct device for a particular platform.
  AudioDevice *audioDevice = AudioDeviceFactory::buildAudioDevice();

  // The device capabilities class will be filled out with information on what
  // the low level device supports.
  DeviceCaps *deviceCaps = AudioDevice::getDevCaps();

  // Here you could have some logic that uses the deviceCaps class to figure
  // out what the device supports and choose what settings are most desired
  // from settings which are available. For this example application the 
  // capabilities will be printed out.
  deviceCaps->print();

  // Here all the DeviceCaps class is told what settings the device should
  // use when it is opened.
  deviceCaps->setInputDeviceToUse(0);
  deviceCaps->setSampleRateToUse(0,AudioFormat::S22K);
  deviceCaps->setEncodingToUse(0,AudioFormat::PCM);
  deviceCaps->setChannelToUse(0,AudioFormat::STEREO);
  deviceCaps->setNumBitsToUse(0,AudioFormat::B8);
  deviceCaps->setBlockSize(0,311);

  // Set the out device to something a little different.
  deviceCaps->setOutputDeviceToUse(1);
  deviceCaps->setSampleRateToUse(1,AudioFormat::S44K);
  deviceCaps->setEncodingToUse(1,AudioFormat::PCM);
  deviceCaps->setChannelToUse(1,AudioFormat::MONO);
  deviceCaps->setNumBitsToUse(1,AudioFormat::B16);
  deviceCaps->setBlockSize(1,311);

  // use a large number of blocks to show that the buffer
  // does not get filled.
  int numBlocks = 200;

  // make sure the buffer size is different than the block
  // size set earlier, to exersize the read function's
  // ability to use different buffer sizes.
  AudioDevice::sample_t *buffer = new AudioDevice::sample_t[512*numBlocks];
  char buff[256];

  // Open and close the device a few times to make sure
  // everything is working and no memory leaks are caused.
  for(int i = 0; i < 5;i++)
  {
    audioDevice->open(deviceCaps);

    cout << "device opened" << endl;

    if(audioDevice->isOpen())
    {
      audioDevice->close();
      cout << "device closed" << endl;
    }
    else
    {
      cout << "device not open, you can not close it" << endl;
    }

    cout << "read thread killed " << endl;
  }

  cout << "quitting opening and closing test" << endl;

  // open the device and read the device
  // for a while to make sure read is
  // working properly.
  cout << "type go to start read test" << endl;
  cin >> buff;
  while(!(strcmp(buff,"stop") == 0))
  {
    audioDevice->open(deviceCaps);
    cout << "device opened" << endl;
    cout << "waiting for device to fill buffer" << endl;
    // wait a few seconds for the other 
    // tread to fill the buffer.
    sleep(3000);

    cout << "bytes read from device " << audioDevice->read(buffer,512*numBlocks) << endl; 
    cout << "waiting for device to fill buffer" << endl;
    // wait a few seconds for the other 
    // tread to fill the buffer.
    sleep(3000);

    if(audioDevice->isOpen())
    {
      audioDevice->close();
      cout << "device closed" << endl;
      cout << "read thread killed " << endl;
    }
    else
    {
      cout << "device not open, you can not close it" << endl;
    }

    cout << "type stop to end read test" << endl;
    cin >> buff;
  }

  cout << "quitting reading test" << endl;

  // open the device and read the device
  // for a while to make sure read is
  // working properly.
  cout << "type go to start read/write test" << endl;
  cin >> buff;
  strcpy(buff,"reset");

  // set the out device to the same format as the in device
  deviceCaps->setOutputDeviceToUse(1);
  deviceCaps->setSampleRateToUse(0,AudioFormat::S22K);
  deviceCaps->setEncodingToUse(0,AudioFormat::PCM);
  deviceCaps->setChannelToUse(0,AudioFormat::STEREO);
  deviceCaps->setNumBitsToUse(0,AudioFormat::B8);
  deviceCaps->setBlockSize(1,311);

  int bytesRead = 0;
  int bytesWritten = 0;
  int inputVol = 0;
  int outputVol = 0;
  while(!(strcmp(buff,"stop") == 0))
  {
    audioDevice->open(deviceCaps);
    cout << "device opened" << endl;
    cout << "what should the input volume be (0-100)?" << endl;
    cin >> inputVol;
    audioDevice->setInputVol(inputVol);
    cout << "input volume set to " << audioDevice->getInputVol() << endl;
    cout << "what should the output volume be (0-100)?" << endl;
    cin >> outputVol;
    audioDevice->setOutputVol(outputVol);
    cout << "output volume set to " << audioDevice->getOutputVol() << endl;
    cout << "waiting for device to fill buffer" << endl;
    // wait a few seconds for the other 
    // tread to fill the buffer.
    sleep(3000);

    bytesRead = audioDevice->read(buffer,512*numBlocks);
    cout << "bytes read from device " << bytesRead << endl;
    bytesWritten = audioDevice->write(buffer,bytesRead);
    cout << "bytes written to device " << bytesWritten << endl;
    cout << "waiting for device to fill buffer" << endl;
    // wait a few seconds for the other 
    // tread to fill the buffer.
    sleep(3000);

    if(audioDevice->isOpen())
    {
      audioDevice->close();
      cout << "device closed" << endl;
      cout << "read thread killed " << endl;
    }
    else
    {
      cout << "device not open, you can not close it" << endl;
    }

    cout << "type stop to end read/write test" << endl;
    cin >> buff;

  }

  cout << "quitting read/write test" << endl;

  return 0;
}


