// Copyright 2001 by Columbia University. All rights reserved.

// This class stores information about one audio device. It is used by
// DeviceCaps as the basic unit of information about a device. Information
// about the interogated device is stored here. The library's user should
// not really have to be concerned with this class.

#ifndef DEVCAP_H
#define DEVCAP_H

#include <string>
#include <Common/AudioFormat.h>
#include <Common/AudioFormats.h>

using namespace std;

// A handle to a DevCap
typedef int DevHandle_t;

class DevCap
{
  public:

  static const int NONE;

  DevCap();
  DevCap(const DevCap &right);
  ~DevCap();

  // sets the handle of this DevCap
  void setDevHandle(DevHandle_t handle);

  // returns the handle of this DevCap
  DevHandle_t getDevHandle();

  // Sets/gets the name of the device. The device's name is set by the
  // platform's device according to the platform's device layer.
  void setDevName(string *name);
  string* getDevName();

  // Sets/gets the AudioFormats class of this DevCaps. The platform's device
  // is interogated by the NAL's platform independent device layer.
  void setAudioFormats(AudioFormats *pAudioFormats);
  AudioFormats* getAudioFormats();

  // Input and output devices can set their block size seperatley.
  void setBlockSize(int blockSize);
  int getBlockSize();

  // Set to true, if the device supports input volume.
  // Set to false, if the device does not support input volume.
  void setInVolSupp(bool inVolSupp);

  // Returns true, if the device supports input volume.
  // Returns false, if the device does not support input volume.
  bool getInVolSupp();

  // Set to true, if the device supports output volume.
  // Set to false, if the device does not support output volume.
  void setOutVolSupp(bool outVolSupp);

  // Returns true, if the device supports output volume.
  // Returns false, if the device does not support output volume.
  bool getOutVolSupp();

  // Refer to the AudioFormat class to see what all of these settings mean.

  void setSampleRateToUse(AudioFormat::SampleRate sampleRate);
  AudioFormat::SampleRate getSampleRateToUse();

  void setEncodingToUse(AudioFormat::Encoding encoding);
  AudioFormat::Encoding getEncodingToUse();

  void setChannelToUse(AudioFormat::Channel channel);
  AudioFormat::Channel getChannelToUse();

  void setNumBitsToUse(AudioFormat::NumBits numBits);
  AudioFormat::NumBits getNumBitsToUse();

  // Prints the contents of a DevCap to standard out.
  void print();

  protected:

  private:

  // The handle to this DevCap.
  DevHandle_t devHandle;

  // The name of according to the platform's device layer.
  string *pName;

  // All the audio formats supported by this DevCap.
  AudioFormats *pAudioFormats;

  // The block size used by this device.
  int blockSize;

  // true, if the device supports input volume, false if not.
  bool inVolSupp;

  // true, if the device supports output volume, false if not.
  bool outVolSupp;

  // The AudioFormat the libary's user sets as the one to use.
  // It is set through funcitons from DeviceCaps.
  AudioFormat settingsToUse;

};

#endif