// Copyright 2001 by Columbia University. All rights reserved.

// stop the VC ++ compiler's complaints about STL
#ifdef WIN32
#pragma warning (disable:4786)
#endif

#include <AudioDevices/AudioDevice.h>
#include <AudioDevices/DeviceCaps.h>

DeviceCaps::DeviceCaps()
{
} // DeviceCaps::DeviceCaps


DeviceCaps::DeviceCaps(const DeviceCaps &right)
{
  devCaps = right.devCaps;
  it = right.it;
} // DeviceCaps::DeviceCaps


DeviceCaps::~DeviceCaps()
{
} // DeviceCaps::~DeviceCaps


int
DeviceCaps::getSize()
{
  Debug D("DeviceCaps::getSize");

  return devCaps.size();
} // DeviceCaps::getSize

void
DeviceCaps::setDevName(DevHandle_t handle, const char *pName)
{
  Debug D("DeviceCaps::setDevName");

  checkHandle(handle);
  it->second.setDevName(new string(pName));
} // DeviceCaps::setDevName


string*
DeviceCaps::getDevName(DevHandle_t handle)
{
  Debug D("DeviceCaps::getDevName");

  it = devCaps.find(handle);
  if (it == devCaps.end()) {
    return NULL;
  }
  else {
    return it->second.getDevName();
  }    
} // DeviceCaps::getDevName


void
DeviceCaps::setAudioFormats(DevHandle_t handle, AudioFormats *pAudioFormats)
{
  Debug D("DeviceCaps::setAudioFormats");

  checkHandle(handle);
  it->second.setAudioFormats(pAudioFormats);
} // DeviceCaps::setAudioFormats


AudioFormats*
DeviceCaps::getAudioFormats(DevHandle_t handle)
{
  it = devCaps.find(handle);
  if (it == devCaps.end()) {
    return NULL;
  }
  else {
    return it->second.getAudioFormats();
  }
} // DeviceCaps::getAudioFormats


void
DeviceCaps::setBlockSize(DevHandle_t handle, int numBytes)
{
  Debug D("DeviceCaps::setBlockSize");

  checkHandle(handle);
  it->second.setBlockSize(numBytes);
} // DeviceCaps::setBlockSize


int
DeviceCaps::getBlockSize(DevHandle_t handle)
{
  Debug D("DeviceCaps::getBlockSize");

  it = devCaps.find(handle);
  return it->second.getBlockSize();
} // DeviceCaps::getBlockSize


void
DeviceCaps::setInVolSupp(DevHandle_t handle, bool supported)
{
  Debug D("DeviceCaps::setInVolSupp");

  checkHandle(handle);
  it->second.setInVolSupp(supported);
} // DeviceCaps::setInVolSupp


void
DeviceCaps::setOutVolSupp(DevHandle_t handle, bool supported)
{
  Debug D("DeviceCaps::setOutVolSupp");

  checkHandle(handle);
  it->second.setOutVolSupp(supported);
} // DeviceCaps::setOutVolSupp


bool
DeviceCaps::getInVolSupp(DevHandle_t handle)
{
  it = devCaps.find(handle);
  if (it == devCaps.end()) {
    return 0;
  }
  else {
    return it->second.getInVolSupp();
  }
} // DeviceCaps::getInVolSupp


bool
DeviceCaps::getOutVolSupp(DevHandle_t handle)
{
  it = devCaps.find(handle);
  if (it == devCaps.end()) {
    return 0;
  }
  else {
    return it->second.getOutVolSupp();
  }
} // DeviceCaps::getOutVolSupp


void
DeviceCaps::checkHandle(DevHandle_t handle)
{
  Debug D("DeviceCaps::checkHandle");

  it = devCaps.find(handle);

  if (devCaps.empty() || it == devCaps.end()) {
    DevCap devCap;
    devCaps[handle] = devCap;
    D.Dbg("handle just inserted ", handle);
    D.Dbg("size ", (int)devCaps.size());
  }

  it = devCaps.find(handle);
} // DeviceCaps::checkHandle


void
DeviceCaps::print()
{
  cout << "size of device caps is " << devCaps.size() << endl;
  for (it = devCaps.begin();it != devCaps.end();it++) {
    it->second.print();
  }
} // DeviceCaps::print


void
DeviceCaps::setInputDeviceToUse(DevHandle_t handle)
{
  inputDeviceToUse = handle;
} // DeviceCaps::setInputDeviceToUse


void
DeviceCaps::setOutputDeviceToUse(DevHandle_t handle)
{
   outPutDeviceToUse = handle;
} // DeviceCaps::setOutputDeviceToUse


DevHandle_t
DeviceCaps::getInputDeviceToUse()
{
  return inputDeviceToUse;
} // DeviceCaps::getInputDeviceToUse


DevHandle_t
DeviceCaps::getOutputDeviceToUse()
{
  return outPutDeviceToUse;
} // DeviceCaps::getOutputDeviceToUse


void
DeviceCaps::setSampleRateToUse(DevHandle_t handle, 
                               AudioFormat::SampleRate sampleRate)
{
  it = devCaps.find(handle);
  it->second.setSampleRateToUse(sampleRate);
} // DeviceCaps::setSampleRateToUse


AudioFormat::SampleRate
DeviceCaps::getSampleRateToUse(DevHandle_t handle)
{
  it = devCaps.find(handle);
  return it->second.getSampleRateToUse();
} // DeviceCaps::getSampleRateToUse


void
DeviceCaps::setEncodingToUse(DevHandle_t handle, 
                             AudioFormat::Encoding encoding)
{
  it = devCaps.find(handle);
  it->second.setEncodingToUse(encoding);
} // DeviceCaps::setEncodingToUse


AudioFormat::Encoding
DeviceCaps::getEncodingToUse(DevHandle_t handle)
{
  it = devCaps.find(handle);
  return it->second.getEncodingToUse();
} // DeviceCaps::getEncodingToUse


void
DeviceCaps::setChannelToUse(DevHandle_t handle, AudioFormat::Channel channel)
{
  it = devCaps.find(handle);
  it->second.setChannelToUse(channel);
} // DeviceCaps::setChannelToUse


AudioFormat::Channel
DeviceCaps::getChannelToUse(DevHandle_t handle)
{
  it = devCaps.find(handle);
  return it->second.getChannelToUse();
} // DeviceCaps::getChannelToUse


void
DeviceCaps::setNumBitsToUse(DevHandle_t handle, AudioFormat::NumBits numBits)
{
  it = devCaps.find(handle);
  it->second.setNumBitsToUse(numBits);
} // DeviceCaps::setNumBitsToUse


AudioFormat::NumBits
DeviceCaps::getNumBitsToUse(DevHandle_t handle)
{
  it = devCaps.find(handle);
  return it->second.getNumBitsToUse();
} // DeviceCaps::getNumBitsToUse