// Copyright 2001 by Columbia University. All rights reserved.

// The device capabilities class.

// This class is used as a device information storage class. It is handed to 
// the low level device and that low level device fills out information in
// this class and hands it back to the caller. The caller then chooses what
// capabilities it wants to initialize the device from the capabilities the
// device communicates are supported (through the DevCaps class). The basic
// unit of storage for this class is a DevCap.

// This class is used by the OSSAudioDevice layer to add functionality
// to the basic DeviceCaps class needed for OSS.

#ifndef OSSDEVICECAPS_H
#define OSSDEVICECAPS_H

#include <AudioDevices/DeviceCaps.h>
#include <AudioDevices/OSSDevCap.h>

class OSSDeviceCaps : public DeviceCaps
{
  public:

  OSSDeviceCaps();
  OSSDeviceCaps(const OSSDeviceCaps &right);
  ~OSSDeviceCaps();

  // None of these functions should be called by the library's user. The
  // OSSAudioDevice uses these functions internally to open its own
  // internal devices.

  // Used to open the wave out device.
  void setAudioDevName(DevHandle_t handle, string* pAudioDevName);
  string* getAudioDevName(DevHandle_t handle);

  // Used to open the mixer device.
  void setMixerDevName(DevHandle_t handle, string* pAudioDevName);
  string* getMixerDevName(DevHandle_t handle);

  protected:

  private:

  // This function makes sure that the requested device handle exists. If
  // it does not, a DevCap is inserted in the DevCapMap with the appropriate
  // handle as a key.
  void checkHandle(DevHandle_t devHandle);

  // An STL map used to store DevCaps.
  typedef map<DevHandle_t,OSSDevCap> DevCapsMap;
  DevCapsMap ossDevCaps;

  // The iterator used to find various devices in the DevCapsMap.
  DevCapsMap::iterator it;

};

#endif
