// Copyright 2001 by Columbia University. All rights reserved.

// This file is so long because it allows the platform device to be
// referenced in one device. Since the win32 API is rather complicated,
// the has to be long.

// stop the VC ++ compiler's complaints about STL
#pragma warning (disable:4786)

#include <assert.h>
#include <AudioDevices/AudioDeviceException.h>
#include <AudioDevices/Win32AudioDevice.h>

const int Win32AudioDevice::MAX_WAVE_VOLUME = 0xffff;
const int Win32AudioDevice::NUM_BLOCKS = 128;
const int Win32AudioDevice::DEFAULT_BLOCK_SIZE = 512;
int Win32AudioDevice::audioFormatKey = 0;

Win32AudioDevice::Win32AudioDevice()
{
  Debug D("Win32AudioDevice::Win32AudioDevice");

  deviceOpen = false;
  pWaveInHdrs = NULL;
  pWaveInBuf = NULL;
  pWaveOutHdrs = NULL;
  pWaveOutBuf = NULL;
  waveInHandle = NULL;
  waveOutHandle = NULL;
  mixerHandle = NULL;
  inputDeviceToUse = -1;
  outputDeviceToUse = -1;
  pFirstWaveHdr = NULL;
  readBlockSize = DEFAULT_BLOCK_SIZE;
  writeBlockSize = DEFAULT_BLOCK_SIZE;
  mixerClosed = true;
  waveInClosed = true;
  waveOutClosed = true;
  pMutex = new Mutex();
} // Win32AudioDevice::Win32AudioDevice


Win32AudioDevice::~Win32AudioDevice()
{
  Debug D("Win32AudioDevice::~Win32AudioDevice");

  if (deviceOpen == true) {
    this->close();
  }
  else {
    D.Dbg("destructor " , "device open on close");
  }
} // Win32AudioDevice::~Win32AudioDevice


DeviceCaps*
Win32AudioDevice::getDevCaps()
{
  Debug D("Win32AudioDevice::getDevCaps");

  Win32DeviceCaps *pDeviceCaps = new Win32DeviceCaps();

  numInWaves = waveInGetNumDevs();
  numOutWaves = waveOutGetNumDevs();
  numMixers = mixerGetNumDevs();

  D.Dbg("number of wave in devices ", (int)numInWaves);
  D.Dbg("number of wave out devices ", (int)numOutWaves);

  int audioDev, waveInDev, waveOutDev;
  audioFormatKey = 0;

  // audioDev starts at 0.
  for (waveInDev = 0, audioDev = 0;audioDev < numInWaves;
       waveInDev++, audioDev++) {

    D.Dbg("audioDev ", audioDev);

    if (getWaveInCaps(waveInDev, audioDev, pDeviceCaps) != true) {
      D.Dbg("could not get in device capabilities", "");
    }
  }

  // audioDev continues from numInWaves up.
  for (waveOutDev = 0;audioDev < numOutWaves+numInWaves;
       waveOutDev++, audioDev++) {
    
    D.Dbg("audioDev ", audioDev);

    if (getWaveOutCaps(waveOutDev, audioDev, pDeviceCaps) != true) {
      D.Dbg("could not get out device capabilities", "");
    }
  }

  numInWaves = 0;
  numOutWaves = 0;
  numMixers = 0;

  pWin32DeviceCaps = pDeviceCaps;

  return pDeviceCaps;

} // Win32AudioDevice::getDevCaps


bool
Win32AudioDevice::getWaveInCaps(int waveInDev, int audioDev, 
                                Win32DeviceCaps *pDeviceCaps)
{
  Debug D("Win32AudioDevice::getWaveInCaps");

  WAVEINCAPS waveInCaps;
  MMRESULT mmresult = waveInGetDevCaps(waveInDev,&waveInCaps,
                                       sizeof(WAVEINCAPS));

  if (mmresult != MMSYSERR_NOERROR) {
    D.Dbg("could not get out device capabilities","");
    return false;
  }

  pDeviceCaps->setDevName(audioDev, waveInCaps.szPname);  
  pDeviceCaps->setWaveInDesc(audioDev, waveInDev);

  D.Dbg("wave in caps name ", pDeviceCaps->getDevName(audioDev)->data());
  D.Dbg("format ", (int)waveInCaps.dwFormats);
  D.Dbg("channels ", waveInCaps.wChannels);

  // This seems to be the only way to relate a mixer with the waveIn device.
  MIXERLINE mixerLine;
  mixerLine.cbStruct = sizeof(mixerLine);
  mixerLine.Target.dwType = MIXERLINE_TARGETTYPE_WAVEIN;
  strncpy(mixerLine.Target.szPname, waveInCaps.szPname, MAXPNAMELEN);
  mixerLine.Target.vDriverVersion = waveInCaps.vDriverVersion;
  mixerLine.Target.wMid = waveInCaps.wMid;
  mixerLine.Target.wPid = waveInCaps.wPid;

  mmresult = mixerGetLineInfo((HMIXEROBJ)waveInDev, &mixerLine,
                              MIXER_OBJECTF_MIXER | 
                              MIXER_GETLINEINFOF_TARGETTYPE);

  if (mmresult == MMSYSERR_NOERROR) {

    pDeviceCaps->setRecLine(audioDev, mixerLine.dwLineID);
    pDeviceCaps->setInVolSupp(audioDev, true);
    D.Dbg("mixer line ID ", (int)mixerLine.dwLineID);
  }
  else {
    D.Dbg("problem getting mixerLineInfo", "");
    return false;
  }

  MIXERCAPS mixerCaps;
  mixerCaps.wMid = waveInCaps.wMid;
  mixerCaps.wPid = waveInCaps.wPid;
  mixerCaps.vDriverVersion = waveInCaps.vDriverVersion;
  strncpy(mixerCaps.szPname,waveInCaps.szPname, MAXPNAMELEN);
  mixerCaps.cDestinations = mixerLine.dwDestination;

  HMIXER mixHandle;

  for (int mixerDev = 0;mixerDev < numMixers;mixerDev++) {

    mmresult = mixerOpen(&mixHandle, mixerDev,
                       (unsigned long)NULL,
                       (unsigned long)NULL,
                       MIXER_OBJECTF_MIXER);
    
    if (mmresult == MMSYSERR_NOERROR) {
      pDeviceCaps->setMixerDesc(audioDev,mixerDev);
      break;
    }
  }

  // Check which sampling modes are supported.
  AudioFormats *pAudioFormats = new AudioFormats();
  for (int rate = 0; rate < NUM_SAMPLE_RATES;rate++) {

    if (NAL2WaveFormat(waveInCaps.dwFormats,sampleRates[rate],
                      AudioFormat::MONO,AudioFormat::B8)) {

      pAudioFormats->setSampleRate(audioFormatKey,sampleRates[rate]);
      setInEncodings(waveInDev,audioFormatKey,pAudioFormats,sampleRates[rate],
                     AudioFormat::MONO,AudioFormat::B8);
      pAudioFormats->setChannel(audioFormatKey,AudioFormat::MONO);
      pAudioFormats->setNumBits(audioFormatKey,AudioFormat::B8);
      audioFormatKey++;

    }
    if (NAL2WaveFormat(waveInCaps.dwFormats,sampleRates[rate],
			                   AudioFormat::MONO,AudioFormat::B16)) {

      pAudioFormats->setSampleRate(audioFormatKey,sampleRates[rate]);
      setInEncodings(waveInDev,audioFormatKey,pAudioFormats,sampleRates[rate],
                     AudioFormat::MONO,AudioFormat::B16);
      pAudioFormats->setChannel(audioFormatKey,AudioFormat::MONO);
      pAudioFormats->setNumBits(audioFormatKey,AudioFormat::B16);
      audioFormatKey++;

    }
    if (NAL2WaveFormat(waveInCaps.dwFormats,sampleRates[rate],
                      AudioFormat::STEREO,AudioFormat::B8)) {

      pAudioFormats->setSampleRate(audioFormatKey,sampleRates[rate]);
      setInEncodings(waveInDev,audioFormatKey,pAudioFormats,sampleRates[rate],
                   AudioFormat::STEREO,AudioFormat::B8);
      pAudioFormats->setChannel(audioFormatKey,AudioFormat::STEREO);
      pAudioFormats->setNumBits(audioFormatKey,AudioFormat::B8);
      audioFormatKey++;

    }
    if (NAL2WaveFormat(waveInCaps.dwFormats,sampleRates[rate],
			                   AudioFormat::STEREO,AudioFormat::B16)) {

      pAudioFormats->setSampleRate(audioFormatKey,sampleRates[rate]);
      setInEncodings(waveInDev,audioFormatKey,pAudioFormats,sampleRates[rate],
                     AudioFormat::STEREO,AudioFormat::B16);
      pAudioFormats->setChannel(audioFormatKey,AudioFormat::STEREO);
      pAudioFormats->setNumBits(audioFormatKey,AudioFormat::B16);
      audioFormatKey++;

    }
  } // close for

  pDeviceCaps->setAudioFormats(audioDev,pAudioFormats);

  return true;

} // Win32AudioDevice::getWaveInCaps


bool
Win32AudioDevice::getWaveOutCaps(int waveOutDev, int audioDev,
                                 Win32DeviceCaps *pDeviceCaps)
{
  Debug D("Win32AudioDevice::getWaveOutCaps");

  WAVEOUTCAPS waveOutCaps;
  MMRESULT mmresult = waveOutGetDevCaps(waveOutDev,&waveOutCaps,
                                        sizeof(WAVEOUTCAPS));

  if (mmresult != MMSYSERR_NOERROR) {
    D.Dbg("","problem getting waveOutCaps");
    return false;
  }

  pDeviceCaps->setDevName(audioDev, waveOutCaps.szPname);

  D.Dbg("wave out caps name ", pDeviceCaps->getDevName(audioDev)->data());
  D.Dbg("format ", (int)waveOutCaps.dwFormats);
  D.Dbg("channels ", waveOutCaps.wChannels);

  // Check which sampling modes are supported.
  AudioFormats *pAudioFormats = new AudioFormats();
  for (int rate = 0; rate < NUM_SAMPLE_RATES;rate++) {

    if (NAL2WaveFormat(waveOutCaps.dwFormats,sampleRates[rate],
                      AudioFormat::MONO,AudioFormat::B8)) {

      pAudioFormats->setSampleRate(audioFormatKey,sampleRates[rate]);
      setOutEncodings(waveOutDev,audioFormatKey,pAudioFormats,sampleRates[rate],
                     AudioFormat::MONO,AudioFormat::B8);
      pAudioFormats->setChannel(audioFormatKey,AudioFormat::MONO);
      pAudioFormats->setNumBits(audioFormatKey,AudioFormat::B8);
      audioFormatKey++;

    }
    if (NAL2WaveFormat(waveOutCaps.dwFormats,sampleRates[rate],
			                   AudioFormat::MONO,AudioFormat::B16)) {

      pAudioFormats->setSampleRate(audioFormatKey,sampleRates[rate]);
      setOutEncodings(waveOutDev,audioFormatKey,pAudioFormats,sampleRates[rate],
                     AudioFormat::MONO,AudioFormat::B16);
      pAudioFormats->setChannel(audioFormatKey,AudioFormat::MONO);
      pAudioFormats->setNumBits(audioFormatKey,AudioFormat::B16);
      audioFormatKey++;

    }
    if (NAL2WaveFormat(waveOutCaps.dwFormats,sampleRates[rate],
                      AudioFormat::STEREO,AudioFormat::B8)) {

      pAudioFormats->setSampleRate(audioFormatKey,sampleRates[rate]);
      setOutEncodings(waveOutDev,audioFormatKey,pAudioFormats,sampleRates[rate],
                   AudioFormat::STEREO,AudioFormat::B8);
      pAudioFormats->setChannel(audioFormatKey,AudioFormat::STEREO);
      pAudioFormats->setNumBits(audioFormatKey,AudioFormat::B8);
      audioFormatKey++;

    }
    if (NAL2WaveFormat(waveOutCaps.dwFormats,sampleRates[rate],
			                   AudioFormat::STEREO,AudioFormat::B16)) {

      pAudioFormats->setSampleRate(audioFormatKey,sampleRates[rate]);
      setOutEncodings(waveOutDev,audioFormatKey,pAudioFormats,sampleRates[rate],
                     AudioFormat::STEREO,AudioFormat::B16);
      pAudioFormats->setChannel(audioFormatKey,AudioFormat::STEREO);
      pAudioFormats->setNumBits(audioFormatKey,AudioFormat::B16);
      audioFormatKey++;

    }
  } // close for

  if (waveOutCaps.dwSupport & WAVECAPS_VOLUME) {
    pDeviceCaps->setOutVolSupp(audioDev,true);
  }

  pDeviceCaps->setAudioFormats(audioDev,pAudioFormats);
  pDeviceCaps->setWaveOutDesc(audioDev,waveOutDev);

  return true;

} // Win32AudioDevice::getWaveOutCaps


void
Win32AudioDevice::setInEncodings(int waveIn,FormatHandle_t handle, 
                                 AudioFormats *pAudioFormats,
                                 AudioFormat::SampleRate rate, 
                                 AudioFormat::Channel channels,
                                 AudioFormat::NumBits bits)
{
  Debug D("Win32AudioDevice::setInEncodings");

  D.Dbg("rate ", (int)rate);
  D.Dbg("channels ", (int)channels);
  D.Dbg("bits ", (int)bits);

  WAVEFORMATEX waveTex;

  // Check which audio encodings are supported.
  for (int encoding = 0;encoding < NUM_ENCODINGS;encoding++) {

    waveTex.wFormatTag      = NAL2WaveEncoding(encodings[encoding]);
    waveTex.nChannels       = channels;
    waveTex.nSamplesPerSec  = rate;
    waveTex.wBitsPerSample  = bits;
    waveTex.nBlockAlign     = waveTex.wBitsPerSample / 8 * waveTex.nChannels;
    waveTex.nAvgBytesPerSec = waveTex.nSamplesPerSec * waveTex.nBlockAlign;
    waveTex.cbSize = 0;

    MMRESULT mmresult = waveInOpen(NULL, (UINT)waveIn, &waveTex, (UINT)NULL,
                                   (UINT)NULL, 
                                   WAVE_MAPPED | WAVE_FORMAT_QUERY);

    if (mmresult != MMSYSERR_NOERROR) {

      if (mmresult == WAVERR_BADFORMAT) {
        D.Dbg("bad format", "");
      }

      D.Dbg("could not query wave in device ", waveIn);
      D.Dbg("format tag ", waveTex.wFormatTag);
      D.Dbg("channels ", waveTex.nChannels);
      D.Dbg("bits per sample ", waveTex.wBitsPerSample);
      char error[256];
      mmresult = waveInGetErrorText(mmresult,error,256);
      D.Dbg("", error);
 
    }
    else {
      pAudioFormats->setEncoding(handle, encodings[encoding]);
    }

    WAVEINCAPS waveInCaps;

    mmresult = waveInGetDevCaps((UINT)waveIn,&waveInCaps,sizeof(waveInCaps));
  } // close for

} // Win32AudioDevice::setInEncodings


void
Win32AudioDevice::setOutEncodings(int waveOut,FormatHandle_t handle,     
                                  AudioFormats *pAudioFormats,
                                  AudioFormat::SampleRate rate, 
                                  AudioFormat::Channel channels,
                                  AudioFormat::NumBits bits)
{
  Debug D("Win32AudioDevice::setOutEncodings");

  D.Dbg("rate ", (int)rate);
  D.Dbg("channels ", (int)channels);
  D.Dbg("bits ", (int)bits);

  WAVEFORMATEX waveTex;

  // Check which audio encodings are supported.
  for (int encoding = 0;encoding < NUM_ENCODINGS;encoding++) {

    waveTex.wFormatTag      = NAL2WaveEncoding(encodings[encoding]);
    waveTex.nChannels       = channels;
    waveTex.nSamplesPerSec  = rate;
    waveTex.wBitsPerSample  = bits;
    waveTex.nBlockAlign     = waveTex.wBitsPerSample / 8 * waveTex.nChannels;
    waveTex.nAvgBytesPerSec = waveTex.nSamplesPerSec * waveTex.nBlockAlign;
    waveTex.cbSize = 0;
      
    MMRESULT mmresult = waveOutOpen(NULL,(UINT)waveOut,&waveTex,(UINT)NULL,
                                    (UINT)NULL, WAVE_MAPPED | WAVE_FORMAT_QUERY);
    
    if (mmresult != MMSYSERR_NOERROR) {

      if (mmresult == WAVERR_BADFORMAT){
        D.Dbg("bad format", "");
      }

      D.Dbg("could not query wave out device", "");
      D.Dbg("format tag ", waveTex.wFormatTag);
      D.Dbg("channels ", waveTex.nChannels);
      D.Dbg("bits per sample ", waveTex.wBitsPerSample);
      char error[256];
      mmresult = waveOutGetErrorText(mmresult,error,256);
      D.Dbg("", error);
 
    }
    else {
      pAudioFormats->setEncoding(handle, encodings[encoding]);
    }

  } // close for

} // Win32AudioDevice::setOutEncodings


int
Win32AudioDevice::NAL2WaveEncoding(AudioFormat::Encoding encoding)
{
  Debug D("Win32AudioDevice::NAL2WaveEncoding");

  D.Dbg("encoding ", (int)encoding);

  switch (encoding) {	
    case AudioFormat::PCM:
      D.Dbg("PCM ","WAVE_FORMAT_PCM");
      return WAVE_FORMAT_PCM;
    case AudioFormat::PCMU:
      D.Dbg("PCMU ", "WAVE_FORMAT_MULAW");
      return WAVE_FORMAT_MULAW;
    case AudioFormat::PCMA:
      D.Dbg("PCMA ", "WAVE_FORMAT_ALAW");
      return WAVE_FORMAT_ALAW;
    case AudioFormat::S8:
      D.Dbg("S8 " , "WAVE_FORMAT_UNKNOWN");
      return WAVE_FORMAT_UNKNOWN;
    case AudioFormat::U8:
      D.Dbg("U8 ", "WAVE_FORMAT_UNKNOWN");
      return WAVE_FORMAT_UNKNOWN;
    case AudioFormat::S16:
      D.Dbg("S16 ", "WAVE_FORMAT_UNKNOWN");
      return WAVE_FORMAT_UNKNOWN;
  };

  return AudioFormat::NO_FORMAT;

} // Win32AudioDevice::NAL2WaveEncoding


bool
Win32AudioDevice::NAL2WaveFormat(DWORD dwFormat, AudioFormat::SampleRate rate, 
                                 AudioFormat::Channel channel, 
                                 AudioFormat::NumBits bits)
{
  Debug D("Win32AudioDevice::NAL2WaveFormat");

  D.Dbg("format ", (int)dwFormat);
  D.Dbg("sample rate ", (int)rate);
  D.Dbg("channel ", (int)channel);
  D.Dbg("bits ", (int)bits);

  if (channel == AudioFormat::MONO) {
    if (rate == AudioFormat::S11K) {
      if (bits == AudioFormat::B8) {       
        // WAVE_FORMAT_1M08 11.025 kHz, Mono, 8-bit
        if (WAVE_FORMAT_1M08 & dwFormat) {
          return true;
        }
      }
      else if (bits == AudioFormat::B16) {      
        // WAVE_FORMAT_1M16 11.025 kHz, Mono, 16-bit
        if (WAVE_FORMAT_1M16 & dwFormat) {
          return true;
        } 
      }
    } // close rate
    else if (rate == AudioFormat::S22K) {
      if (bits == AudioFormat::B8) {
        // WAVE_FORMAT_2M08 22.05 kHz, Mono, 8-bit
        if (WAVE_FORMAT_2M08 & dwFormat) {
          return true;
        }
      }
      else if (bits == AudioFormat::B16) {
        // WAVE_FORMAT_2M16 22.05 kHz, Mono, 16-bit
        if (WAVE_FORMAT_2M16 & dwFormat) {
          return true;
        }
      }
    } // close rate
    else if (rate == AudioFormat::S44K) {
      if (bits == AudioFormat::B8) {
        // WAVE_FORMAT_4M08 44.1 kHz, Mono, 8-bit
        if (WAVE_FORMAT_4M08 & dwFormat) {
          return true;
        }
      }
      else if (bits == AudioFormat::B16) {
        // WAVE_FORMAT_4M16 44.1 kHz, Mono, 16-bit
        if (WAVE_FORMAT_4M16 & dwFormat) {
          return true;
        }
      }
    } // close rate
  } // close channel
  else if (channel == AudioFormat::STEREO) {
    if (rate == AudioFormat::S11K) { 
      if (bits == AudioFormat::B8) {
        // WAVE_FORMAT_1S08 11.025 kHz, Stereo, 8-bit
        if (WAVE_FORMAT_1S08 & dwFormat) {
          return true;
        }
      }
      else if (bits == AudioFormat::B16) {
        // WAVE_FORMAT_1S16 11.025 kHz, Stereo, 16-bit
        if (WAVE_FORMAT_1S16 & dwFormat) {
          return true;
        }
      }
    } // close rate
    else if (rate == AudioFormat::S22K) {
      if (bits == AudioFormat::B8) {
        // WAVE_FORMAT_2S08 22.05 kHz, Stereo, 8-bit
        if (WAVE_FORMAT_2S08 & dwFormat) {
          return true;
        }
      }
      else if (bits == AudioFormat::B16) {
        // WAVE_FORMAT_2S16 22.05 kHz, Stereo, 16-bit
        if (WAVE_FORMAT_2S16 & dwFormat) {
          return true;
        }
        return true;
      }
    } // close rate
    else if (rate == AudioFormat::S44K) {
      if (bits == AudioFormat::B8) {
        // WAVE_FORMAT_4S08 44.1 kHz, Stereo, 8-bit
        if (WAVE_FORMAT_4S08 & dwFormat) {
          return true;
        }
      }
      else if (bits == AudioFormat::B16) {
        // WAVE_FORMAT_4S16 44.1 kHz, Stereo, 16-bit
        if (WAVE_FORMAT_4S16 & dwFormat) {
          return true;
        }
      }
    } // close rate
  } // close channel
  else {
    // WAVE_INVALIDFORMAT invalid format 
    return false;
  }

  return false;

} // Win32AudioDevice::NAL2WaveFormat


void
Win32AudioDevice::open(DeviceCaps *deviceCaps)
{
  Debug D("Win32AudioDevice::open");

  Win32DeviceCaps *pWin32DeviceCaps = (Win32DeviceCaps*)deviceCaps;

  inputDeviceToUse = pWin32DeviceCaps->getInputDeviceToUse();
  outputDeviceToUse = pWin32DeviceCaps->getOutputDeviceToUse();

  if (deviceOpen == true) {
    D.Dbg("the win32 device is already open", "");
    throw AudioDeviceException("the win32 device is already open");
  }
  else {
    deviceOpen = true;
  }

  // we want to make sure the device is opened
  // before the read thread starts to mess with
  // various data structures.
  pMutex->acquire();

  readThread = CreateThread(NULL,NULL,
                           (LPTHREAD_START_ROUTINE)Win32AudioDevice::readProc,
                           this,NULL,&readThreadID);

  try {
    openWaveIn();
    D.Dbg("could not open the wave in device", "");
    
  }
  catch (AudioDeviceException *ex) {
    throw AudioDeviceException(ex->what());
  }

  pMutex->release();

  try {
    openMixer();
    D.Dbg("could not open the mixer device", "");
  } 
  catch (AudioDeviceException *ex) {
    throw AudioDeviceException(ex->what());
  }

  try {
    openWaveOut();
    D.Dbg("could not open the wave out device", "");
  } 
  catch (AudioDeviceException *ex) {
    ex->what();
    throw AudioDeviceException(ex->what());
  }

} // Win32AudioDevice::open


// Setup a buffer and open the wave in device.
// This device records sound from the microphone. 
void
Win32AudioDevice::openWaveIn()
{
  Debug D("Win32AudioDevice::openWaveIn");

  if (waveInHandle != NULL) {
    D.Dbg("the wave input device is already open", "");
    throw AudioDeviceException("the wave input device is already open");
  }

  readBlockSize = pWin32DeviceCaps->getBlockSize(inputDeviceToUse);

  WAVEFORMATEX waveTex;
  waveTex.wFormatTag = NAL2WaveEncoding(
                       pWin32DeviceCaps->getEncodingToUse(inputDeviceToUse));
  waveTex.nChannels = pWin32DeviceCaps->getChannelToUse(inputDeviceToUse);
  waveTex.nSamplesPerSec = pWin32DeviceCaps->getSampleRateToUse(inputDeviceToUse);
  waveTex.wBitsPerSample = pWin32DeviceCaps->getNumBitsToUse(inputDeviceToUse);
  waveTex.nBlockAlign = waveTex.wBitsPerSample / 8 * waveTex.nChannels;
  waveTex.nAvgBytesPerSec = waveTex.nSamplesPerSec * waveTex.nBlockAlign;
  waveTex.cbSize = 0;

  // setup buffers and release if necessary
  if (pWaveInHdrs != NULL) {
    delete pWaveInHdrs;
  }

  pWaveInHdrs = new WAVEHDR[NUM_BLOCKS];

  if (pWaveInBuf != NULL) {
    delete pWaveInBuf;
  }

  pWaveInBuf = new char[NUM_BLOCKS*readBlockSize];

  MMRESULT mmresult = waveInOpen(&waveInHandle,
                                 pWin32DeviceCaps->getWaveInDesc(inputDeviceToUse),
                                 &waveTex, readThreadID, NULL, CALLBACK_THREAD);

  waveInGetErrorText(mmresult,errorText,MAXERRORLENGTH);

  D.Dbg(errorText, "");

  // set the waveInHdrs to point into wavInBuf
  for (int i = 0; i < NUM_BLOCKS; i++) {

    pWaveInHdrs[i].lpData = pWaveInBuf + i * readBlockSize;
    pWaveInHdrs[i].dwBufferLength = readBlockSize;
    pWaveInHdrs[i].dwFlags = 0;
    mmresult = waveInPrepareHeader(waveInHandle,&pWaveInHdrs[i],
                                   sizeof(WAVEHDR));
    assert(mmresult == MMSYSERR_NOERROR);
    mmresult = waveInAddBuffer(waveInHandle,&pWaveInHdrs[i],
                               sizeof(WAVEHDR));
    assert(mmresult == MMSYSERR_NOERROR);

  }

  pFirstWaveHdr = NULL;
  int err = waveInStart(waveInHandle);

  if (err != 0) {
    waveOutGetErrorText(err, errorText, sizeof(errorText));
    D.Dbg("waveOutGetErrorText ", errorText);
    exit(1);
  }

  waveInClosed = false;

} // Win32AudioDevice::openWaveIn


// Setup a buffer and open the wave out device.
// This device plays sound to the speakers. For
// now only 16 bit audio format is supported.
void
Win32AudioDevice::openWaveOut()
{
  Debug D("Win32AudioDevice::openWaveOut");

  writeBlockSize = pWin32DeviceCaps->getBlockSize(outputDeviceToUse);

  if (waveOutHandle != NULL) {
    D.Dbg("the wave input device is already open","");
    throw AudioDeviceException("the wave input device is already open");
  }

  WAVEFORMATEX waveTex;
  waveTex.wFormatTag = NAL2WaveEncoding(
                            pWin32DeviceCaps->getEncodingToUse(outputDeviceToUse));
  waveTex.nChannels = pWin32DeviceCaps->getChannelToUse(outputDeviceToUse);
  waveTex.nSamplesPerSec = pWin32DeviceCaps->getSampleRateToUse(outputDeviceToUse);
  waveTex.wBitsPerSample = pWin32DeviceCaps->getNumBitsToUse(outputDeviceToUse);
  waveTex.nBlockAlign = waveTex.wBitsPerSample / 8 * waveTex.nChannels;
  waveTex.nAvgBytesPerSec = waveTex.nSamplesPerSec * waveTex.nBlockAlign;
  waveTex.cbSize = 0;
  
  // setup buffers and release if necessary
  if (pWaveOutHdrs != NULL) {
    delete pWaveOutHdrs;
  }

  pWaveOutHdrs = new WAVEHDR[NUM_BLOCKS];

  if (pWaveOutBuf != NULL) {
    delete pWaveOutBuf;
  }

  pWaveOutBuf = new char[NUM_BLOCKS*writeBlockSize];

  MMRESULT mmresult = waveOutOpen(&waveOutHandle,
                                  pWin32DeviceCaps->getWaveOutDesc(outputDeviceToUse), 
                                  &waveTex, NULL, NULL, CALLBACK_NULL);

  // get new buffer for each wave header
  for (int i = 0; i < NUM_BLOCKS; i++) {

    pWaveOutHdrs[i].dwFlags = 0;
    pWaveOutHdrs[i].dwBufferLength = writeBlockSize;
    pWaveOutHdrs[i].lpData = pWaveOutBuf + i * writeBlockSize;
    pWaveOutHdrs[i].dwUser = i;
    mmresult = waveOutPrepareHeader(waveOutHandle,&pWaveOutHdrs[i],
                                    sizeof(WAVEHDR));
    pWaveOutHdrs[i].dwFlags |= WHDR_DONE;
    assert(mmresult == MMSYSERR_NOERROR);
  }

  waveOutClosed = false; 
}


void
Win32AudioDevice::openMixer()
{
  Debug D("Win32AudioDevice::openMixer");

  if (mixerClosed == false) {
    closeMixer();
    mixerHandle = 0;
  }

  MMRESULT mmresult = mixerOpen(&mixerHandle,
                                pWin32DeviceCaps->getWaveInDesc(inputDeviceToUse),
                                (unsigned long)NULL,(unsigned long)NULL,
                                MIXER_OBJECTF_MIXER);

  if (mmresult != MMSYSERR_NOERROR) {
    D.Dbg("could not open mixer","");
    throw AudioDeviceException("could not open mixer");
  }

} // Win32AudioDevice::openWaveOut


bool
Win32AudioDevice::isOpen() { return deviceOpen; }


void
Win32AudioDevice::close()
{
  Debug D("Win32AudioDevice::close");

  // need to protect the data structures
  // so the read thread does not corrupt
  // things
  pMutex->acquire();

  try {
    closeWaveIn();
  }
  catch (AudioDeviceException ex) {
    throw AudioDeviceException("could not close wave in device");
  }

  pMutex->release();

  try {
    closeMixer();
  }
  catch (AudioDeviceException ex) {
    throw AudioDeviceException("could not close wave mixer device");
  }

  try {
    closeWaveOut();
  }
  catch (AudioDeviceException ex) {
    throw AudioDeviceException("could not close wave out device");
  }

  if (deviceOpen == true) {
    deviceOpen = false;
    D.Dbg("closed win32 audio device", "");
  }
  else {
    D.Dbg("could not properly close win32 audio device", "");
  }

} // Win32AudioDevice::close


void
Win32AudioDevice::closeWaveIn()
{
  Debug D("Win32AudioDevice::closeWaveIn");

  if (waveInHandle == NULL) {
    throw AudioDeviceException("the device is not open");
  }

  if (PostThreadMessage(readThreadID,WM_QUIT,0,0) == 0) {

    DWORD error = GetLastError();
    pMutex->acquire();
    D.Dbg("could not kill read thread", "");
    pMutex->release();
  }
  else {

    pMutex->acquire();
    D.Dbg("read thread killed ", "");
    pMutex->release();
  }

  waveInReset(waveInHandle);

  for (int i = 0;i < NUM_BLOCKS;i++) {

    if (pWaveInHdrs[i].dwFlags & WHDR_PREPARED) {
      waveInUnprepareHeader(waveInHandle,&pWaveInHdrs[i],
			                         sizeof(WAVEHDR));
    }
    else {
      D.Dbg("a WAVEHDR was not prepared properly", "");
    }
  }

  waveInClose(waveInHandle);

  delete pWaveInHdrs;
  pWaveInHdrs = NULL;
  delete pWaveInBuf;
  pWaveInBuf = NULL;
  pFirstWaveHdr = NULL;
  waveInClosed = true;
  waveInHandle = NULL;
  inputDeviceToUse = -1;

} // Win32AudioDevice::closeWaveIn


void
Win32AudioDevice::closeWaveOut()
{
  Debug D("Win32AudioDevice::closeWaveOut");

  if (waveOutHandle == NULL) {
    throw AudioDeviceException("the wave out device is not open");
  }

  waveOutReset(waveOutHandle);

  for (int i = 0;i < NUM_BLOCKS;i++) {

    if (pWaveOutHdrs[i].dwFlags & WHDR_PREPARED) {
      waveOutUnprepareHeader(waveOutHandle,&pWaveOutHdrs[i],
			     sizeof(WAVEHDR));
    }
    else {
      D.Dbg("a WAVEHDR was not prepared properly","");
    }
  }

  waveOutClose(waveOutHandle);

  delete pWaveOutHdrs;
  pWaveOutHdrs = NULL;
  delete pWaveOutBuf;
  pWaveOutBuf = NULL;
  waveOutClosed = true;
  waveOutHandle = NULL;
  outputDeviceToUse = -1;

} // Win32AudioDevice::closeWaveOut

int
Win32AudioDevice::read(sample_t *pAppBuff, int appBuffSize)
{
  Debug D("Win32AudioDevice::read");

  if (pAppBuff == NULL) {
    return NULL;
  }

  // Since buffSize may not be a multiple of NUK_BLOCKS, 
  // the size of the last (or only) block must determined.
  int lastBlockSize = appBuffSize % readBlockSize;
  int restBuffSize = appBuffSize - lastBlockSize;
  int bytesCopied = 0;

  WAVEHDR *pDevWaveHdr; // used to hand a waveheader to device
  MMRESULT mmresult;

  int blah = 0;

  // copy everything of size bytesPerBlock to device
  while (pFirstWaveHdr != NULL && bytesCopied < restBuffSize) {

    pDevWaveHdr = pFirstWaveHdr;
    // Step though the wave headers pointing into the waveInBuf and
    // copy the wave headers' data into the wave device's buffers.
    memcpy(pAppBuff,pDevWaveHdr->lpData,readBlockSize);
    pFirstWaveHdr = pFirstWaveHdr->lpNext;
    assert(pDevWaveHdr->dwFlags & WHDR_DONE);
    assert(pDevWaveHdr->dwFlags & ~WHDR_INQUEUE);
    pDevWaveHdr->lpNext          = NULL; 
    pDevWaveHdr->dwBytesRecorded = 0; 
    pDevWaveHdr->dwFlags        &= ~WHDR_DONE;
    mmresult = waveInAddBuffer(waveInHandle, pDevWaveHdr, sizeof(WAVEHDR)); 
    assert(mmresult == MMSYSERR_NOERROR);
    blah++;
    bytesCopied += readBlockSize;
  }

  // copy the last block (or the only block) to device
  if (pFirstWaveHdr != NULL && lastBlockSize != NULL) {

    pDevWaveHdr = pFirstWaveHdr;
    memcpy(pAppBuff,pDevWaveHdr->lpData,lastBlockSize);
    pFirstWaveHdr = pFirstWaveHdr->lpNext;
    assert(pDevWaveHdr->dwFlags & WHDR_DONE);
    assert(pDevWaveHdr->dwFlags & ~WHDR_INQUEUE);
    pDevWaveHdr->lpNext          = NULL; 
    pDevWaveHdr->dwBytesRecorded = 0; 
    pDevWaveHdr->dwFlags        &= ~WHDR_DONE;
    mmresult = waveInAddBuffer(waveInHandle,pDevWaveHdr, sizeof(WAVEHDR));
    assert(mmresult == MMSYSERR_NOERROR);
    bytesCopied += lastBlockSize;
  }

  return bytesCopied;

} // Win32AudioDevice::read


void
Win32AudioDevice::readProc(Win32AudioDevice *win32Dev)
{
  win32Dev->waveInProc();
} // Win32AudioDevice::readProc


void
Win32AudioDevice::waveInProc()
{
  // no debug used here, so it does not
  // have to be protected in the other
  // thread.

  MSG msg;
  
  WAVEHDR *pNewWaveHdr;
  WAVEHDR **pWaveHdrToInsert;

  while (true) {

    GetMessage(&msg,NULL,0,0);
    
    pMutex->acquire();

    // stop this thread
    if (msg.message == WM_QUIT) {
      break;
    }

    if (msg.message == MM_WIM_DATA) {
      // the new wave header from the device
      pNewWaveHdr = (WAVEHDR*)msg.lParam;
      pNewWaveHdr->lpNext = NULL;

      // start at the begging of the list
      pWaveHdrToInsert = &pFirstWaveHdr;

      // loop through the list to find the place
      // to insert the new wave header
      while (*pWaveHdrToInsert != NULL) {
        pWaveHdrToInsert = &((*pWaveHdrToInsert)->lpNext);
      } // close MM_WIM_DATA

      // insert the new wave header at the end of the list
      *pWaveHdrToInsert = pNewWaveHdr;
    } // close while

    pMutex->release();

  }

} // Win32AudioDevice::waveInProc


int
Win32AudioDevice::write(sample_t *pAppBuff, int appBuffSize)
{
  Debug D("Win32AudioDevice::write");

  if (pAppBuff == NULL) {
    return NULL;
  }

  // Since buffSize may not be a multiple of NUM_BLOCKS, the size of
  // the last (or only) block must determined.
  int lastBlockSize = appBuffSize % writeBlockSize;
  int restBuffSize = appBuffSize - lastBlockSize;

  WAVEHDR *pDevWaveHdr = NULL; // used to hand a waveheader to device
  
  MMRESULT mmresult;
  int bytesCopied = 0;

  // the buffer is larger than one block 
  if (writeBlockSize < appBuffSize ) {

    while (bytesCopied < restBuffSize) {

      pDevWaveHdr = this->getOutHdr();

      if (pDevWaveHdr == NULL) {
        D.Dbg("ran out of space in win32device buffer", "");
        break;
      }

      memcpy(pAppBuff,pDevWaveHdr->lpData,writeBlockSize);
      bytesCopied += writeBlockSize;
      mmresult = waveOutWrite(waveOutHandle,pDevWaveHdr,sizeof(WAVEHDR));

      waveOutGetErrorText(mmresult, errorText, MAXERRORLENGTH);

      if (mmresult == WAVERR_STILLPLAYING) {
	       break;
      }
    } // close while

    if (lastBlockSize != NULL) {

      memcpy(pAppBuff,pDevWaveHdr->lpData,lastBlockSize);
      bytesCopied += lastBlockSize;
      mmresult = waveOutWrite(waveOutHandle,pDevWaveHdr,sizeof(WAVEHDR));
    } // close lastBlockSize

  } // close writeBlockSize
  // the buffer is smaller than one block
  else {

    pDevWaveHdr = this->getOutHdr();
    memcpy(pAppBuff,pDevWaveHdr->lpData,lastBlockSize);
    bytesCopied += lastBlockSize;
    mmresult = waveOutWrite(waveOutHandle,pDevWaveHdr,sizeof(WAVEHDR));
  }

  return bytesCopied;

} // Win32AudioDevice::write


WAVEHDR*
Win32AudioDevice::getOutHdr()
{
  Debug D("Win32AudioDevice::getOutHdr");

  WAVEHDR *waveHdr = NULL;

  for (int i = 0;i < NUM_BLOCKS;i++) {

    assert(pWaveOutHdrs[i].dwFlags & WHDR_PREPARED);

    if (pWaveOutHdrs[i].dwFlags & WHDR_DONE) {
      pWaveOutHdrs[i].dwFlags &= WHDR_PREPARED;
      waveHdr = &pWaveOutHdrs[i];
      break;
    }
  }

  return waveHdr;

} // Win32AudioDevice::getOutHdr


void
Win32AudioDevice::closeMixer()
{
  Debug D("Win32AudioDevice::closerMixer");

  // a possible enhancement could be to save
  // the mixer settings in use before the
  // library opened the mixer.

  MMRESULT mmresult;
  mmresult = mixerClose(mixerHandle);

  if (mmresult != MMSYSERR_NOERROR) {
    D.Dbg("could not close mixer", "");
    throw AudioDeviceException("could not close mixer");
  }

  mixerClosed = true;

} // Win32AudioDevice::closeMixer

int
Win32AudioDevice::vol2Dev(int volume)
{
  Debug D("vol2Dev");
  D.Dbg("new volume will be set to " , volume);
  return ((volume * MAX_WAVE_VOLUME / MAX_VOLUME) << 16 |
	  (volume * MAX_WAVE_VOLUME / MAX_VOLUME));
}


int
Win32AudioDevice::dev2Vol(int vol)
{
  Debug D("Win32AudioDevice::dev2Vol");
  D.Dbg("new volume will be set to " , vol);

  return ((vol & 0xffff) * MAX_VOLUME / MAX_WAVE_VOLUME);

} // Win32AudioDevice::vol2Dev


void
Win32AudioDevice::setInputVol(int vol)
{
  Debug D("Win32AudioDevice::setInputVol");
  D.Dbg("new volume will be set to ", vol);

  MIXERCONTROLDETAILS_UNSIGNED gain;
  MIXERCONTROLDETAILS mixerCtrlDetails;
  MIXERLINECONTROLS mixerLineCtrls;
  MIXERCONTROL mixerCtrl;
  MMRESULT mmresult;

  mixerLineCtrls.cbStruct = sizeof(mixerLineCtrls);
  mixerLineCtrls.pamxctrl = &mixerCtrl;
  mixerLineCtrls.cbmxctrl = sizeof(MIXERCONTROL);
  // this is causing a problem with setting the input volume.
  mixerLineCtrls.dwLineID = pWin32DeviceCaps->getRecLine(inputDeviceToUse);
  mixerLineCtrls.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;

  mmresult = mixerGetLineControls((HMIXEROBJ)pWin32DeviceCaps->getMixerDesc(inputDeviceToUse),
                                  &mixerLineCtrls,
                                  MIXER_GETLINECONTROLSF_ONEBYTYPE |
                                  MIXER_OBJECTF_HMIXER);

  waveOutGetErrorText(mmresult, errorText, sizeof(errorText));

  if (mmresult != MMSYSERR_NOERROR) {
    D.Dbg("could not get mixer line controls", errorText);
  }

  mixerCtrlDetails.cbStruct = sizeof(mixerCtrlDetails);
  mixerCtrlDetails.dwControlID = mixerCtrl.dwControlID;
  mixerCtrlDetails.cChannels = 1;
  mixerCtrlDetails.cMultipleItems = mixerCtrl.cMultipleItems;
  mixerCtrlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_BOOLEAN);
  mixerCtrlDetails.paDetails = &gain;
  gain.dwValue = vol;

  mmresult = mixerSetControlDetails((HMIXEROBJ)pWin32DeviceCaps->getMixerDesc(inputDeviceToUse),
                                    &mixerCtrlDetails,
                                    MIXER_OBJECTF_HMIXER);

  waveOutGetErrorText(mmresult, errorText, sizeof(errorText));

  if (mmresult != MMSYSERR_NOERROR) {
    D.Dbg("could not set mixer controls details", errorText);
  }

} // Win32AudioDevice::setInputVol


int
Win32AudioDevice::getInputVol()
{
  Debug D("Win32AudioDevice::getInputVol");

  MIXERCONTROLDETAILS_UNSIGNED uMixerCtrlDetails;
  MIXERCONTROLDETAILS mixerCtrlDetails;
  MIXERLINECONTROLS mixerLineCtrl;
  MIXERCONTROL mixerCtrl;
  MMRESULT mmresult;

  mixerLineCtrl.cbStruct = sizeof(mixerLineCtrl);
  mixerLineCtrl.pamxctrl = &mixerCtrl;
  mixerLineCtrl.cbmxctrl = sizeof(MIXERCONTROL);
  mixerLineCtrl.dwLineID = pWin32DeviceCaps->getRecLine(inputDeviceToUse);
  mixerLineCtrl.dwControlType = MIXERCONTROL_CONTROLTYPE_VOLUME;

  mmresult = mixerGetLineControls((HMIXEROBJ)mixerHandle, &mixerLineCtrl,
                                  MIXER_GETLINECONTROLSF_ONEBYTYPE |
                                  MIXER_OBJECTF_HMIXER);

  if (mmresult != MMSYSERR_NOERROR) {
    return 0;
  }

  mixerCtrlDetails.cbStruct = sizeof(mixerCtrlDetails);
  mixerCtrlDetails.dwControlID = mixerCtrl.dwControlID;
  mixerCtrlDetails.cChannels = 1;
  mixerCtrlDetails.cMultipleItems = mixerCtrl.cMultipleItems;
  mixerCtrlDetails.cbDetails = sizeof(MIXERCONTROLDETAILS_UNSIGNED);
  mixerCtrlDetails.paDetails = &uMixerCtrlDetails;

  mmresult = mixerGetControlDetails((HMIXEROBJ)mixerHandle, 
                                    &mixerCtrlDetails,
                                    MIXER_OBJECTF_HMIXER);

  if (mmresult != MMSYSERR_NOERROR) {
    return 0;
  }

  return (int)(uMixerCtrlDetails.dwValue * MAX_WAVE_VOLUME /
	      (mixerCtrl.Bounds.dwMaximum - mixerCtrl.Bounds.dwMinimum));

} // Win32AudioDevice::getInputVol


void
Win32AudioDevice::setOutputVol(int vol)
{
  Debug D("setOutputVol");

  D.Dbg("new volume will be set to ", vol);

  if (waveOutHandle == NULL) {
    D.Dbg("", "tried to set volume without opening device");
    return;
  }

  DWORD volume = vol2Dev(vol);
  int err = waveOutSetVolume(waveOutHandle,volume);

  if (err == 0) {
    waveOutGetErrorText(err, errorText, sizeof(errorText));
    D.Dbg("error during waveInStart ", errorText);
  }

} // Win32AudioDevice::setOutputVol


int
Win32AudioDevice::getOutputVol()
{
  Debug D("Win32AudioDevice::getOutputVol");

  if (waveOutHandle == NULL) {
    D.Dbg("", "tried to set volume without opening device");
    return NULL;
  }

  DWORD vol = NULL;

  int err = waveOutGetVolume(waveOutHandle,&vol);

  if (err == 0) {
    waveOutGetErrorText(err, errorText, sizeof(errorText));
    D.Dbg("error during waveInStart " , errorText);
    return NULL;
  }

  return dev2Vol(vol);

} // Win32AudioDevice::getOutputVol


AudioFormat::Channel
Win32AudioDevice::getNumChannels()
{
  Debug D("Win32AudioDevice::getNumChannels");

  return (AudioFormat::Channel)0;

} // Win32AudioDevice::getNumChannels


AudioFormat::SampleRate
Win32AudioDevice::getSampleRate()
{
  Debug D("Win32AudioDevice::getSampleRate");

  return (AudioFormat::SampleRate)0;
} // Win32AudioDevice::getSampleRate


int
Win32AudioDevice::getReadBlockSize()
{
  Debug D("Win32AudioDevice::getReadBlockSize");

  return readBlockSize;

} // Win32AudioDevice::getReadBlockSize


int
Win32AudioDevice::getWriteBlockSize()
{
  Debug D("Win32AudioDevice::getReadBlockSize");

  return writeBlockSize;

} // Win32AudioDevice::getWriteBlockSize


void
Win32AudioDevice::printHdrStats(WAVEHDR *waveHdrs)
{

  if (waveHdrs->dwFlags & WHDR_DONE) {
      cout << "wave header done" << endl;
  }

  if (waveHdrs->dwFlags & WHDR_INQUEUE) {
      cout << "wave header inqueue" << endl;
  }

  if (waveHdrs->dwFlags & WHDR_PREPARED) {
      cout << "wave header prepared" << endl;
  }

} // Win32AudioDevice::printHdrStats