// Copyright 2001 by Columbia University. All rights reserved.

// The device capabilities class.

// This class is used as a device information storage class. It is handed to 
// the low level device and that low level device fills out information in
// this class and hands it back to the caller. The caller then chooses what
// capabilities it wants to initialize the device from the capabilities the
// device communicates are supported (through the DevCaps class). The basic
// unit of storage for this class is a DevCap.

// This class is used by the win32AudioDevice driver to add functionality
// to the basic DeviceCaps class needed for win32.

#ifndef WIN32DEVICECAPS_H
#define WIN32DEVICECAPS_H

#include <AudioDevices/DeviceCaps.h>
#include <AudioDevices/Win32DevCap.h>

class Win32DeviceCaps : public DeviceCaps
{
  public:

  Win32DeviceCaps();
  Win32DeviceCaps(const Win32DeviceCaps &right);
  ~Win32DeviceCaps();

  // None of these functions should be called by the library's user. The
  // Win32AudioDevice uses these functions internally to open its own
  // internal devices.

  // The Recline is used for input volume.
  void setRecLine(DevHandle_t handle, DWORD recLine);
  DWORD getRecLine(DevHandle_t handle);

  // Used to open the wave in device.
  void setWaveInDesc(DevHandle_t handle, UINT waveInDesc);
  UINT getWaveInDesc(DevHandle_t handle);

  // Used to open the wave out device.
  void setWaveOutDesc(DevHandle_t handle, UINT waveOutDesc);
  UINT getWaveOutDesc(DevHandle_t handle);

  // Used to open the mixer device.
  void setMixerDesc(DevHandle_t handle, UINT mixerDesc);
  UINT getMixerDesc(DevHandle_t handle);

  protected:

  private:

  // This function makes sure that the requested device handle exists. If
  // it does not, a DevCap is inserted in the DevCapMap with the appropriate
  // handle as a key.
  void checkHandle(DevHandle_t devHandle);

  // An STL map used to store DevCaps.
  typedef map<DevHandle_t,Win32DevCap> DevCapsMap;
  DevCapsMap win32DevCaps;

  // The iterator used to find various devices in the DevCapsMap.
  DevCapsMap::iterator it;

};

#endif