// WAVAudioFile.cpp: implementation of the WAVAudioFile class.
//
//////////////////////////////////////////////////////////////////////

#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

#include "WAVAudioFile.h"

#include <iostream>

#ifdef WIN32
#include <mmreg.h>
#else
const int WAVE_FORMAT_GSM610 = 0x0031;
const int WAVE_FORMAT_DVI_ADPCM = 0x0011;
const int WAVE_FORMAT_ALAW = 0x0006;
const int WAVE_FORMAT_MULAW = 0x0007;
#endif

using namespace std;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

WAVAudioFile::WAVAudioFile(istream &stream)
:AudioFile(stream)
{

  Debug D("WAVAudioFile::WAVAudioFile");
  
  readToDataChunk();

  initAudioFormat();

}

WAVAudioFile::~WAVAudioFile()
{

}

AudioFormat::Encoding WAVAudioFile::intToEncoding(int wFormatTag)
{
  
  AudioFormat::Encoding encoding;
  
  switch(wFormatTag){
  case WAVE_FORMAT_GSM610:
  case WAVE_FORMAT_DVI_ADPCM:
  case WAVE_FORMAT_ALAW :
  case WAVE_FORMAT_MULAW:
    encoding = AudioFormat::S16;
    break;
  }
  
  return encoding;
  
}

void WAVAudioFile::readToDataChunk()
{
  Debug D("WAVAudioFile::readToDataChunk");

  char buf[5];

  stream.get(buf,5);
  filesize = *(int*)buf;

  D.Dbg("medium" ,"File Size: ",filesize);

  stream.get(buf,5);
  wavstring = buf;

  D.Dbg("medium","Wave tag: ",wavstring.c_str());

  stream.get(buf,5);
  fmtchunk = buf;

  D.Dbg("medium","Chunk Name: ",fmtchunk.c_str());

  stream.get(buf,5);
  fmtchunksize = *(int*)buf;

  D.Dbg("medium","Chunk Size: ",fmtchunksize);

  wFormatTag = 0;
  stream.get((char*)&wFormatTag,3);

  D.Dbg("medium","Format Tag: ", wFormatTag);

  nChannels = 0;
  stream.get((char*)&nChannels,3);

  D.Dbg("medium","Number of channels: ",nChannels);

  nSamplesPerSec = 0;
  stream.get((char*)&nSamplesPerSec,5);

  D.Dbg("medium","Samples per Second: ",nSamplesPerSec);

  nAvgBytesPerSec = 0;
  stream.get((char*)&nAvgBytesPerSec,5);

  D.Dbg("medium","Avg Bytes Per Second: ",nAvgBytesPerSec);

  D.Dbg("low","Length of sound: ", filesize/nAvgBytesPerSec);

  nBlockAlign = 0;
  stream.get((char*)&nBlockAlign,3);

  D.Dbg("medium","Block Align: ",nBlockAlign);

  wBitsPerSample = 0;
  stream.get((char*)&wBitsPerSample,3);

  D.Dbg("medium","Bits per sample: ",wBitsPerSample);

  cbSize = 0;
  stream.get((char*)&cbSize,3);

  D.Dbg("medium","Remaining Chunk data: ",cbSize);

  if(cbSize > 0){

    stream.get(buf,cbSize + 1);
  }

  if(wFormatTag != WAVE_FORMAT_PCM){

    stream.get(buf,5);
    int dwChunkSize;

    stream.get((char*)&dwChunkSize,5);

    stream.get(buf,dwChunkSize + 1);

  }

  stream.get(buf,5);
  datastring = buf;

  D.Dbg("medium","Data Tag: ",datastring.c_str());

  stream.get(buf,5);

  D.Dbg("medium","Data Length: ",*(int*)buf);

}

void WAVAudioFile::initAudioFormat()
{

  Debug D("WAVAudioFile::initAudioFormat");
  
  audioFormat->setSampleRate(AudioFormat::intToSampleRate(nSamplesPerSec));
  audioFormat->setEncoding(intToEncoding(wFormatTag));
  audioFormat->setChannel(AudioFormat::intToChannel(nChannels));
  audioFormat->setBytesPerBlock(160 * 4);
  audioFormat->setNumBits(AudioFormat::intToNumBits(16));
}

void WAVAudioFile::read(char *buf, int &length)
{

  char *inbuf = new char[length];

  switch(this->wFormatTag){
  case WAVE_FORMAT_DVI_ADPCM:
    stream.read(inbuf,length);
    break;
  default:
    break;
  }

  length = stream.gcount();

}
