// Simple test program for Codecs Library
//
// CodecTest.exe
// Asks for codec to use, input file, and output file.
// Prints formats supported by input codec.
// Reads input file, decodes it, then reencdes it using specified codec
// Writes file.
// 
// Copyright (c) 2001 by Columbia University. All rights reserved.
//////////////////////////////////////////////////////////////////

#ifdef _MSC_VER
#pragma warning(disable:4786)
#endif

#include <winsock2.h>
#include <Codecs/CodecFactory.h>
#include <Codecs/Codec.h>
#include <Common/CodecFormat.h>
#include <iostream>
#include <fstream>
#include <vector>
#include <Utils/Debug.h>

#include "AudioFileFactory.h"

using namespace std;

// Returns the enumerated encoding
AudioFormat::Encoding returnEncoding(int enc, AudioFormat::NumBits &samplebits, int &blocksize){
  AudioFormat::Encoding encoding;
  
  switch(enc){
  case 1:
    encoding = AudioFormat::PCMU;
    samplebits = AudioFormat::B16;
    blocksize = 160 * NUM_BYTES_PER_SAMPLE;
    break;
  default:
    break;
  }
  return encoding;
}

void convertFile(AudioFile &audioFile, ostream &outsndfile,Codec &codec)
{
  Debug D("convertFile");

  vector<unsigned short*> outbufvec;
  vector<unsigned char*> inbufvec;

  CodecFormat* codecFormat = codec.getCodecFormat();
  
  const unsigned int BUFFERLENGTH = codecFormat->getFrameSize();
  int length;
  int samplesread = 0;
  
  unsigned char *inbuf = NULL;
  
  unsigned short *outbuf = NULL;
  
  unsigned char *diskbuf = new unsigned char[BUFFERLENGTH];

  //read the file contents
  while(!audioFile.eof()){

    //this is how much data to read
    length = BUFFERLENGTH;

    //create new buffers
    outbuf = new unsigned short[BUFFERLENGTH];
    inbuf = new unsigned char[BUFFERLENGTH];
  
    //read next frame, byte actually read passed through length
    audioFile.read((char *)inbuf,length);

    //record number of samples read, this is equal to number of bytes
    samplesread += length;
  
    //decode frame
    codec.decode(inbuf,outbuf,length);

    //store buffers
    outbufvec.push_back(outbuf);
    inbufvec.push_back(inbuf);

    codec.encode(outbuf,diskbuf,length);

    //compare buffers
    if(memcmp(inbuf,diskbuf,length) == 0){
      D.Dbg("low","Buffers are identical: ",codec.getCodecFormat()->getName());
    }
    else{
      D.Dbg("low","Buffers are different",codec.getCodecFormat()->getName());
    }

  }

  D.Dbg("medium","Samples read: ",samplesread);


  CodecPtr dviCodec = CodecFactory::instance().create("dvi");
  CodecFormatList codecFormatList = dviCodec->getSupportedFormats();
  dviCodec->setCodecFormat(codecFormatList.front());

  vector<unsigned char*> codedbufvec;
  unsigned char* codedbuf;

  int bufnum = 0;

  while(samplesread > 0){

    if(samplesread > BUFFERLENGTH){
      length = BUFFERLENGTH;
    }
    else{
      length = samplesread;
    }

    samplesread -= length;

    codedbuf = new unsigned char[dviCodec->getCodecFormat()->getFrameSize()];

    unsigned short* tmpbuf = outbufvec[bufnum];

    unsigned short* sbuf = new unsigned short[length];

    dviCodec->encode(tmpbuf,codedbuf,length);
    dviCodec->resetState();

    codedbufvec.push_back(codedbuf);
    
    dviCodec->decode(codedbuf,sbuf,length);
    dviCodec->resetState();
    

    //compare buffers
    if(memcmp(tmpbuf,sbuf,length) == 0){
      D.Dbg("low","Buffers are identical","");
    }

  }
  
  /*
  codec.encode(outbuf,diskbuf,length);
  
  if(memcmp(inbuf,diskbuf,length) == 0){
    D.Dbg("low","Buffers are identical","");
  }

  outsndfile.write((char*)diskbuf,length);

  */

}

void main(){
  
  Debug::specify("D:T:");
  
  string codecname;
  
  //get the codec name from the user
  cout << "Which codec should I use?" << endl;
  cin >>  codecname;
  
  // get instance of the codec
  CodecPtr codec = CodecFactory::instance().create(codecname);
  
  // get and print the supported formats
  CodecFormatList fl = codec->getSupportedFormats();
  
  CodecFormatList::iterator i;
  
  cout << "Supported Formats" << endl;
  
  for(i = fl.begin();i != fl.end();i++){
    AudioFormat &af = i->getAudioFormat();
    cout << i->getDescription() << endl;
    cout << af.getSampleRate() << endl;
  }
  
  // get the audio sound file name
  string filename;
  cout << "Enter sound file name" << endl;
  cin >> filename;
  
  //open the file and read it
  ifstream insndfile(filename.c_str(),ios::in | ios::binary);

  AudioFile* audioFile = AudioFileFactory::buildAudioFile(insndfile);

  AudioFormat af = audioFile->getAudioFormat();
  
  CodecFormat *codecFormat = NULL;
  
  int result = 0;
  for(i = fl.begin();i != fl.end();i++){
    AudioFormat &af2 = i->getAudioFormat();
    
    int tmpRes = af & af2;
    
    if(tmpRes > result){
      result = tmpRes;
      codecFormat = new CodecFormat(*i);
    }
  }
  
  cout << codecFormat->getDescription() 
    << codecFormat->getFrameSize() << endl
    << codecFormat->getAudioFormat().getEncoding() << endl;
  
  
  codec->setCodecFormat(*codecFormat);
  
  cout << "Enter output filename: " << endl;
  cin >> filename;
  
  ofstream outsndfile(filename.c_str(),ios::out | ios::binary);
  
  convertFile(*audioFile,outsndfile,*codec);
  
}