// CodecFactory.h: Contains typedefs to make hide reused factory.
//
// To use factory simple add the following line to codec class:
//
//   namespace{
//    static registerInFactory<Codec, DVI> registerdvi("dvi");
//   }
//
// The codec can then be created using the following code:
//
//   CodecPtr dviCodec = CodecFactory::instance().create("dvi");
//
// Notice that the return value is an STL auto_ptr.
// 
// Copyright (c) 2001 by Columbia University. All rights reserved.
//////////////////////////////////////////////////////////////////

#if !defined (_CODECFACTORY_H_)
#define _CODECFACTORY_H_

#include <Utils/factory.h>
#include <memory>
#include "Codec.h"
#include "g711.h"
#include "g726.h"
#include "dvi.h"

// Convenience typedef's
typedef std::auto_ptr<Codec> CodecPtr;
typedef genericFactory<Codec> CodecFactory;

#endif /* #if !defined (_CODECFACTORY_H_) */
