// GSM.cpp: implementation of the GSM class.
//
// Copyright (c) 2001 by Columbia University. All rights reserved.
//////////////////////////////////////////////////////////////////////

#ifdef _MSC_VER
#pragma warning(disable:4786) //disables nasty vc++ warning caused by stl
#endif

#include "GSM.h"
#include "GSMState.h"

#include "cx_gsm.h"

// GSM::GSM()
// Constructor.  Initializes format list, creates gsm state.
GSM::GSM()
{

  //initialize the formats that this codec supports
  formatlist.insert(formatlist.end(),
    CodecFormat(
      "GSM-8K-Mono",
      "Full rate GSM speech codec. (c) 1992 J. Degener and C. Bormann, Technische Universitaet Berlin.",
      3,0,GSM_FRAMESIZE,
      AudioFormat(AudioFormat::S16,AudioFormat::S8K,AudioFormat::B16,
      AudioFormat::MONO,
				160 * NUM_BYTES_PER_SAMPLE
      )
    )
  );
  
  formatlist.insert(formatlist.end(),
    CodecFormat(
      "GSM-16K-Mono",
      "Full rate GSM speech codec. (c) 1992 J. Degener and C. Bormann, Technische Universitaet Berlin.",
      118,0,GSM_FRAMESIZE,
      AudioFormat(AudioFormat::S16,AudioFormat::S16K,AudioFormat::B16,
      AudioFormat::MONO,
				160 * NUM_BYTES_PER_SAMPLE
      )
    )
  );

  formatlist.insert(formatlist.end(),
    CodecFormat(
      "GSM-32K-Mono",
      "Full rate GSM speech codec. (c) 1992 J. Degener and C. Bormann, Technische Universitaet Berlin.",
      119,0,GSM_FRAMESIZE,
      AudioFormat(AudioFormat::S16,AudioFormat::S32K,AudioFormat::B16,
      AudioFormat::MONO,
				160 * NUM_BYTES_PER_SAMPLE
      )
    )
  );

  formatlist.insert(formatlist.end(),
    CodecFormat(
      "GSM-48K-Mono",
      "Full rate GSM speech codec. (c) 1992 J. Degener and C. Bormann, Technische Universitaet Berlin.",
      123,0,GSM_FRAMESIZE,
      AudioFormat(AudioFormat::S16,AudioFormat::S48K,AudioFormat::B16,
      AudioFormat::MONO,
				160 * NUM_BYTES_PER_SAMPLE
      )
    )
  );

  //create the state variable
  codecState = new GSMState();
}//GSM::GSM

//GSM::~GSM()
//destructor
GSM::~GSM()
{
  
}//GSM::~GSM

// GSM::decode(unsigned char *inbuf, unsigned short *outbuf, int &length)
// Simply calls gsm decode function.
void GSM::decode(unsigned char *inbuf, unsigned short *outbuf, int &length)
{ 
  gsm_decode(
    (gsm)codecState->getState(), 
    (gsm_byte*)inbuf, 
    (gsm_signal*)outbuf
  );

  length = codecFormat->getAudioFormat().getBytesPerBlock() / 
    (codecFormat->getAudioFormat().getNumBits() / 8);
}//GSM::decode(unsigned char *inbuf, unsigned short *outbuf, int &length)

// GSM::encode(unsigned short *inbuf, unsigned char *outbuf, int &length)
// Simply calls gsm encode function.
void GSM::encode(unsigned short *inbuf, unsigned char *outbuf, int &length)
{
  
  gsm_encode((gsm)codecState->getState(), (short *)inbuf, (gsm_byte*)outbuf);
  length =  codecFormat->getFrameSize();
  
}//GSM::encode(unsigned short *inbuf, unsigned char *outbuf, int &length)

// GSM::resetState()
// Deallocates and recreates gsmstate.
void GSM::resetState()
{

  delete codecState;
  codecState = new GSMState();

}//GSM::resetState
