// dvi.cpp: Implementation of the DVI codec.
//
// Copyright (c) 2001 by Columbia University. All rights reserved.
//////////////////////////////////////////////////////////////////

#ifdef _MSC_VER
#pragma warning(disable:4786) //disables nasty vc++ warning caused by stl
#endif

#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#endif

#include "dvi.h"
#include "cx_dvi.h"
#include "DVIState.h"

#include <Utils/Debug.h>

#include <iostream>

using namespace std;

// DVI::DVI()
// Constructor.  Initializes the formats supported by DVI codec and the state
// used for decoding and encoding.
DVI::DVI()
{
  Debug D("DVI::DVI()");
  
  // 20 ms
  formatlist.insert(formatlist.end(),
    CodecFormat(
    "DVI-8K-Mono",
    "IMA ADPCM codec. (c) 1992 Stichting Mathematisch Centrum, Amsterdam, Netherlands.",
    5,4,80,
    AudioFormat(AudioFormat::S16,AudioFormat::S8K,AudioFormat::B16,
    AudioFormat::MONO,
				160 * NUM_BYTES_PER_SAMPLE
        )
        )
        );
  
  // 14 ms
  formatlist.insert(formatlist.end(),
    CodecFormat(
    "DVI-11K-Mono",
    "IMA ADPCM codec. (c) 1992 Stichting Mathematisch Centrum, Amsterdam, Netherlands.",
    16,4,80,
    AudioFormat(AudioFormat::S16,AudioFormat::S11K,AudioFormat::B16,
    AudioFormat::MONO,
				160 * NUM_BYTES_PER_SAMPLE
        )
        )
        );
  
  // 10 ms
  formatlist.insert(formatlist.end(),
    CodecFormat(
    "DVI-16K-Mono",
    "IMA ADPCM codec. (c) 1992 Stichting Mathematisch Centrum, Amsterdam, Netherlands.",
    6,4,80,
    AudioFormat(AudioFormat::S16,AudioFormat::S16K,AudioFormat::B16,
    AudioFormat::MONO,
				160 * NUM_BYTES_PER_SAMPLE
        )
        )
        );
  
  // 7 ms
  formatlist.insert(formatlist.end(),
    CodecFormat(
    "DVI-22K-Mono",
    "IMA ADPCM codec. (c) 1992 Stichting Mathematisch Centrum, Amsterdam, Netherlands.",
    17,4,80,
    AudioFormat(AudioFormat::S16,AudioFormat::S22K,AudioFormat::B16,
    AudioFormat::MONO,
				160 * NUM_BYTES_PER_SAMPLE
        )
        )
        );
  
  // 5 ms
  formatlist.insert(formatlist.end(),
    CodecFormat(
    "DVI-32K-Mono",
    "IMA ADPCM codec. (c) 1992 Stichting Mathematisch Centrum, Amsterdam, Netherlands.",
    81,4,80,
    AudioFormat(AudioFormat::S16,AudioFormat::S32K,AudioFormat::B16,
    AudioFormat::MONO,
				160 * NUM_BYTES_PER_SAMPLE
        )
        )
        );
  
  // 3.3 ms
  formatlist.insert(formatlist.end(),
    CodecFormat(
    "DVI-48K-Mono",
    "IMA ADPCM codec. (c) 1992 Stichting Mathematisch Centrum, Amsterdam, Netherlands.",
    82,4,80,
    AudioFormat(AudioFormat::S16,AudioFormat::S48K,AudioFormat::B16,
    AudioFormat::MONO,
				160 * NUM_BYTES_PER_SAMPLE
        )
        )
        );

  codecState = new DVIState();
}//DVI::DVI

// DVI::decode(unsigned char *inbuf, unsigned short *outbuf, int &length)
// Prepares buffer to be sent to adpcm_decoder function
void DVI::decode(unsigned char *inbuf, unsigned short *outbuf, int &length)
{
  Debug D("DVI::decode()");
  
  adpcm_state *state = (adpcm_state*)codecState->getState();
  
  state->valprev = ntohs(state->valprev);
  
  length = codecFormat->getAudioFormat().getBytesPerBlock() / sizeof(short);
  adpcm_decoder(inbuf, (short*)outbuf, length, state);
}//DVI::decode(unsigned char *inbuf, unsigned short *outbuf, int &length)

// DVI::encode(unsigned short *inbuf, unsigned char *outbuf, int &length)
// Prepares buffer to be sent to adpcm_coder function
void DVI::encode(unsigned short *inbuf, unsigned char *outbuf, int &length)
{
  Debug D("DVI::encode()");

  D.Dbg("HIGH","inbuf",(int)inbuf);
  D.Dbg("HIGH","outbuf",(int)outbuf);
  D.Dbg("HIGH","length",length);

  int samples;
  
  //get the state for this codec
  adpcm_state *state = (adpcm_state*)codecState->getState();
  
  /* Fix coded state for byte ordering */
  state->valprev = htons(state->valprev);
  
  //calculate the number of samples, and length in bytes of output buffer
  length = codecFormat->getFrameSize();
  samples = length * 2; // 80 bytes per frame, 2 samples per byte = 160 samples
  
  adpcm_coder((short *)inbuf, outbuf, samples, state);
}//DVI::encode(unsigned short *inbuf, unsigned char *outbuf, int &length)

// DVI::resetState()
// Deletes, then reallocates the DVI state
void DVI::resetState()
{
  delete codecState;
  codecState = new DVIState();
}//DVI::resetState
