// g726.h: Definition of G726 class.  Implements the
// abstract Codec interface.  Function comments may be found in Codec.h
// 
// Copyright (c) 2001 by Columbia University. All rights reserved.
//////////////////////////////////////////////////////////////////
#if !defined(_G726_H_)
#define _G726_H_

#include "Codec.h"
#include <Utils/factory.h>

using namespace std;

//number of samples in a single G726 frame.
const int G726_SAMPLES_PER_FRAME = 160;

class G726 : public Codec
{
public :
  //Functions defined in Codec.  See Codec.h for documentation.
	virtual void resetState();
  virtual void 
    encode(unsigned short *inbuf, unsigned char *outbuf, int &length);
  virtual void 
    decode(unsigned char *inbuf, unsigned short *outbuf, int &length);

  //constructor
  G726();

  //destructor
  ~G726();
  
protected:
  int pack(unsigned char *buf, unsigned char *cw, 
      unsigned char num_cw, int bps);
  int unpack(unsigned char *cw, unsigned char *buf, 
      unsigned char num_cw, int bps);
  
private:
  
};


//Register this codec with factory.
namespace{
  static registerInFactory<Codec, G726> registerG726("G726");
  static registerInFactory<Codec, G726> registerg726("g726");
}

#endif //!defined(_G726_H_)
