// Copyright 2001 year by Columbia University. All rights reserved.

// stop the compiler's complaints about STL
#ifdef WIN32
#pragma warning (disable:4786)
#endif

#include <Common/AudioFormats.h>

const int NO_FORMAT = -1;

AudioFormats::AudioFormats()
{
} // AudioFormats::AudioFormats


AudioFormats::AudioFormats(const AudioFormats &right)
{
  it = right.it;
  audioFormats = right.audioFormats;
} // AudioFormats::AudioFormats


AudioFormats::~AudioFormats()
{
} // AudioFormats::~AudioFormats


int
AudioFormats::getSize()
{
  Debug D("AudioFormats::getSize");

  return audioFormats.size();
} // AudioFormats::getSize


void
AudioFormats::checkHandle(FormatHandle_t handle)
{
  Debug D("AudioFormats::checkHandle");

  it = audioFormats.find(handle);

  if (audioFormats.empty() || it == audioFormats.end()) {

    AudioFormat audioFormat;
    audioFormats[handle] = audioFormat;
    D.Dbg("handle just inserted ", handle);
    D.Dbg("size ", (int)audioFormats.size());
  }

  it = audioFormats.find(handle);
} // AudioFormats::checkHandle


void
AudioFormats::setEncoding(FormatHandle_t handle, AudioFormat::Encoding encoding)
{
  Debug D("AudioFormats::setEncoding");

  checkHandle(handle);
  it->second.setEncoding(encoding);
} // AudioFormats::setEncoding


AudioFormat::Encoding
AudioFormats::getEncoding(FormatHandle_t handle)
{
  Debug D("AudioFormats::getEncoding");

  it = audioFormats.find(handle);
  return it->second.getEncoding();
} // AudioFormats::getEncoding


void
AudioFormats::setSampleRate(FormatHandle_t handle, AudioFormat::SampleRate sampleRate)
{
  Debug D("AudioFormats::setSampleRate");

  checkHandle(handle);
  it->second.setSampleRate(sampleRate);
} // AudioFormats::setSampleRate


AudioFormat::SampleRate
AudioFormats::getSampleRate(FormatHandle_t handle)
{
  Debug D("AudioFormats::getSampleRate");

  it = audioFormats.find(handle);
  return it->second.getSampleRate();
} // AudioFormats::getSampleRate


void
AudioFormats::setChannel(FormatHandle_t handle, AudioFormat::Channel channel)
{
  Debug D("AudioFormats::setChannel");  

  checkHandle(handle);
  it->second.setChannel(channel);
} // AudioFormats::setChannel


AudioFormat::Channel
AudioFormats::getChannel(FormatHandle_t handle)
{
  it = audioFormats.find(handle);
  return it->second.getChannel();
} // AudioFormats::getChannel


void
AudioFormats::setNumBits(FormatHandle_t handle, AudioFormat::NumBits numBits)
{
  Debug D("AudioFormats::setNumBits");  

  checkHandle(handle);
  it->second.setNumBits(numBits);
} // AudioFormats::setNumBits


AudioFormat::NumBits
AudioFormats::getNumBits(FormatHandle_t handle)
{
  Debug D("AudioFormats::getNumBits");

  it = audioFormats.find(handle);
  return it->second.getNumBits();
} // AudioFormats::getNumBits


void
AudioFormats::print()
{
  cout << "size of audio formats is " << audioFormats.size() << endl;

  for (it = audioFormats.begin(); it != audioFormats.end();it++) {

    cout << "sample rate " << it->second.getSampleRate() << endl;    
    cout << "encoding " << it->second.getEncoding() << endl;
    cout << "channel " << it->second.getChannel() << endl;
    cout << "number of bits " << it->second.getNumBits() << endl;
  }
} // AudioFormats::print