// Copyright 2001 year by Columbia University. All rights reserved.

// The audio format will allow one to figure out what is supported by a
// device/codec. The question of what can be supported can be asked for every
// sample rate. So a question like:
// "Does sample rate 11025khz support 16Bit PCMU with STEREO?"
// can be asked.

#ifndef AUDIOFORMATS_H_INCLUDED
#define AUDIOFORMATS_H_INCLUDED

#include <map>
#include <Common/AudioFormat.h>

using namespace std;

class AudioFormats
{
  public:

  AudioFormats();
  AudioFormats(const AudioFormats &right);
  ~AudioFormats();
  int getSize();

  // Each sample rate can be MONO or STEREO and can support several different
  // encondigs.
  void setSampleRate(FormatHandle_t handle, AudioFormat::SampleRate sampleRate);
  AudioFormat::SampleRate getSampleRate(FormatHandle_t handle);

  void setEncoding(FormatHandle_t handle, AudioFormat::Encoding encoding);
  AudioFormat::Encoding getEncoding(FormatHandle_t handle);

  void setChannel(FormatHandle_t handle, AudioFormat::Channel channel);
  AudioFormat::Channel getChannel(FormatHandle_t handle);

  void setNumBits(FormatHandle_t handle, AudioFormat::NumBits numBits);
  AudioFormat::NumBits getNumBits(FormatHandle_t handle);

  void setBytesPerBlock(FormatHandle_t handle, int blocksize);
  int getBytesPerBlock(FormatHandle_t handle);

  // Use these functions to indicate to the device which encoding should be
  // used for this run of the session (this time the device is opened). The
  // set functions return false if what is asked for is not supported.
  void print();

  protected:

  private:
  
  void checkHandle(FormatHandle_t handle);

  typedef map<int,AudioFormat> AudioFormatMap;

  AudioFormatMap audioFormats;
  AudioFormatMap::iterator it;

};

#endif