// CodecFormat.h: interface for the CodecFormat class.
//
// This class defines the format produced by a particular codec.
// It contains the format of the audio data that may be processed into
// the codec format.
//
// Copyright (c) 2001 by Columbia University. All rights reserved.
//////////////////////////////////////////////////////////////////////

#if !defined(_CODECFORMAT_H_)
#define _CODECFORMAT_H_

#include <string>

using namespace std;

#include <Utils/Debug.h>
#include "AudioFormat.h"

//Represents the format of the encoded data.
class CodecFormat  
{
public:
  
  //Constructor.  Create a new codec format instance.
  //@param name Short name for this codec format.
  //@param description Detailed description for this codec format.
  //@param defaultPt
  //@param stateSize Size in bytes of the state used for decoding/encoding to this codec format.
  //@param frameSize Size in bytes of a single formatted frame of data.
  //@param audioFormat The unencoded audio format this codec supports.
  CodecFormat(string name, string description, char defaultPt, 
      short stateSize, short frameSize, const AudioFormat &audioFormat);

  //Copy constructor
  //@param instance to copy
  CodecFormat(const CodecFormat &right);

  //destructor
  virtual ~CodecFormat();

  //Accessor methods
  string getName(){return name;}
  string getDescription(){return description;}
  char getDefaultPt(){return defaultPt;}
  short getStateSize(){return stateSize;}
  short getFrameSize(){return frameSize;}
  AudioFormat &getAudioFormat(){return *audioFormat;}
  
  //Sets a new audio format.
  //@param audioFormat The new format to support.
  virtual void setAudioFormat(AudioFormat &audioFormat){
    delete this->audioFormat;
    this->audioFormat = new AudioFormat(audioFormat);
  }
  
protected:
  
  //short name of this codec
  string name;

  //longer descrption, usually includes copy right
  string description;

  char defaultPt;

  //number of bytes in state structure
  short stateSize;

  //number of bytes in a sample frame
  short frameSize;

  //format of the audio this codec supports enciding from.
  //typically 16bit linear PCM
  AudioFormat *audioFormat;
  
private:
};

#endif // !defined(_CODECFORMAT_H_)
