// RTPEvent.h: interface for the RTPEvent class.
//
//	Wraps the necessary information to process an RTP event in 
//	a convient class.
//
// Copyright (c) 2001 by Columbia University. All rights reserved.
//////////////////////////////////////////////////////////////////////

#if !defined(_RTPEVENT_H_)
#define _RTPEVENT_H_

#include "RTPCommon.h"



class RTPEvent  
{
public:
  //copy constructor
  RTPEvent(const RTPEvent &right);
  
  //Default Constructor: initializes all members to 0
  RTPEvent();
  
  //Constructor
  //@param cid The RTP Session context id
  //@param opaque The RTP event data, stores pointer
  //@param tp Pointer to a timeval struct
  RTPEvent(int cid,rtp_opaque_t opaque, timeval *tp);
  
  //Destructor, does nothing.
  virtual ~RTPEvent();
  
  //Assignment Operator
  //Copies values from passed RTPEvent, does not allocate new memory
  //@param right Event to copy
  RTPEvent & operator =(const RTPEvent &right);
  
  //@return the time this event is to occur.
  inline double getTime() const {return time;}
  
  //@return the opaque rtp data
  inline rtp_opaque_t getOpaque() const {return opaque;}
  
  //@return The RTP Session context id this event is
  // associated with.
  inline context getContextID() const {return cid;}
  
protected:

  //session context id
  context cid;

  //rtp event data
  rtp_opaque_t opaque;

  //event time
  double time;
  
};

#endif // !defined(_RTPEVENT_H_)
