// RTPException.h: interface for the RTPException class.
// Based on STL exception class.  Used to report errors in the RTP classes.
//
// Copyright (c) 2001 by Columbia University. All rights reserved.
//////////////////////////////////////////////////////////////////////

#if !defined(_RTPEXCEPTION_H_)
#define _RTPEXCEPTION_H_

#include <exception>
#include <string>

using namespace std;

class RTPException : public exception  
{
public:
  
  // Creates a new RTPException instance.
  // @param rtp_error The error code from the RTP Library
  // @param message The error message.
  RTPException(int rtp_error, const char* message);
  
  // Creates a new RTPException instance.
  // @param rtp_error The error code from the RTP Library
  // @param message The error message.
  RTPException(int rtp_error,const string &message);
  
  // Destructor
  virtual ~RTPException();
  
  // Returns the message associated with this exception
  // @return Null terminated string
  virtual const char *what() const throw();
  // Returns the error code associated with this exception
  // @return rtp library error code.
  inline int getRTPErrorCode(){return rtp_error;}
  
protected:
  // The rtp library error code
  int rtp_error;
  
  // Description of why error occurred.
  string message;
  
};


#endif // !defined(_RTPEXCEPTION_H_)
