// Condition.cpp: implementation of the Condition class.
//
// Copyright (c) 2001 by Columbia University. All rights reserved.
//////////////////////////////////////////////////////////////////////

#include "Condition.h"

#include "Semaphore.h"
#include "Mutex.h"
// Condition::Condition(Mutex& mutex)
// Constructor
Condition::Condition(Mutex& mutex)
:mutex(mutex)
{
  
}//Condition::Condition(Mutex& mutex)

// Condition::~Condition()
// Destructor
Condition::~Condition()
{
  
}//Condition::~Condition()

// Condition::signal()
// Release the first semaphore in the semaphore list.
void Condition::signal()
{
  //if thread waiting
  if(semaphoreList.size() > 0){
    
    //get first thread's semaphore
    Semaphore* sema = semaphoreList.front();
    
    //remove from list
    semaphoreList.pop_front();
    
    //signal the thread
    sema->release();
  }
}//Condition::signal()

// Condition::wait()
// Create a new semaphore and wait for it
// to be signaled.
void Condition::wait()
{
  
  //create local semaphore to wait on
  Semaphore sema;
  
  //add to the list of waiting semaphores
  semaphoreList.push_back(&sema);
  
  //don't need this now.
  mutex.release();
  
  //wait here for local cemaphore to be signaled
  sema.acquire();
  
  //reacquire mutex.
  mutex.acquire();
  
}//Condition::wait()

// Condition::wait(int timeout)
// Create a local semaphore and wait for it to
// be signaled or return after timeout has expired.
bool Condition::wait(int timeout)
{
  //create local semaphore to wait on
  Semaphore sema;
  
  //add to the list of waiting semaphores
  semaphoreList.push_back(&sema);
  
  //don't need this now.
  mutex.release();
  
  //wait here for local semaphore to be signaled
  bool ret = sema.acquire(timeout);
  
  //reacquire mutex.
  mutex.acquire();
  
  return ret;
}//Condition::wait(int timeout)

// Condition::signalAll()
// release all semaphores in the semaphore list.
void Condition::signalAll()
{
  
  //if the list is not empty
  if(semaphoreList.size() > 0){
    
    SemaphoreList::iterator itr;
    
    for(itr = semaphoreList.begin();itr != semaphoreList.end();itr++){
      
      Semaphore* sema = *itr;
      sema->release();
      //semaphoreList.erase(itr);
      
    }
    
    //empty the list
    semaphoreList.clear();
    
  }
  
}//Condition::signalAll()
