// Mutex.h: defines Mutex
//
// Provides mutual exculsion synchronization resource.
//
// Example:
//
// Mutex mutex;
// mutex.acquire();
//
// critical section
//
// mutex.release();
//
// Copyright (c) 2001 by Columbia University. All rights reserved.
//////////////////////////////////////////////////////////////////////

#if !defined(_MUTEX_H_)
#define _MUTEX_H_

#ifdef WIN32
#include <windows.h>
#elif POSIX
#include <pthread.h>
#else
#error port me!
#endif

class Mutex  
{
public:
  
  //Release the mutex
  void release();
  
  //Try to acquire the mutex before timeout expires.
  //On platforms other than WIN32 this function attempts
  //to acquire the mutex, and returns immediately
  //@param timeout The number of milliseconds to wait
  //before giving up on acquiring the mutex.
  //@return true if mutex has been acquired, false otherwise.
  bool acquire(int timeout);
  
  //Block until the mutex has been acquired.
  void acquire();

  //Creates a new mutex object.
  Mutex();

  //Destroys this mutex.
  virtual ~Mutex();

protected:
  
  //handle to the os mutex
#ifdef WIN32
  HANDLE hMutex;
#elif POSIX
  pthread_mutex_t hMutex;
#endif
  
};

#endif // !defined(_MUTEX_H_)
