// Semaphore.h: interface for the Semaphore class.
// Provides a traditional semaphore object.  May be initialzed
// with a count.  Use acquire and release to decrement and increment the
// count.
//
// Copyright (c) 2001 by Columbia University. All rights reserved.
//////////////////////////////////////////////////////////////////////

#if !defined(_SEMAPHORE_H_)
#define _SEMAPHORE_H_

#ifdef WIN32
#include <windows.h>
#elif POSIX
#include <pthread.h>
#else
#error port me!
#endif

class Semaphore  
{
public:
  //Wait for the semaphore to be greater than 0, or until the timeout
  //expires.  Decreases the semaphore count by 1.
  //On platforms other than WIN32 this function attempts
  //to acquire the semaphore, and returns immediately
  //@param timeout The number of milliseconds to wait for 
  //this semaphore.
  //@return true if the semaphore has been acquired, false otherwise
  bool acquire(int timeout);
  
  //Signal this semaphore.  This increases the count by one.
  void release(void);
  
  //Wait for the semaphore count to be greater than 0.
  void acquire(void);
  
  //Create a new semaphore.
  //@param count The initial count for this semaphore.
  Semaphore(int count=0);
  virtual ~Semaphore();
  
private:
#ifdef WIN32
  //handle to the os semaphore
  HANDLE hSemaphore;
#endif

#ifdef POSIX
  pthread_mutex_t hMutex; 
  pthread_cond_t hCondition;
  int count;
#endif
};

#endif // !defined(_SEMAPHORE_H_)
