// Thread.cpp: implementation of the Thread class.
//
// Copyright (c) 2001 by Columbia University. All rights reserved.
//////////////////////////////////////////////////////////////////////

#include "Thread.h"

//Add #ifdef for other supported platforms
#ifdef WIN32
#include <process.h>
#elif POSIX
#include <pthread.h>
#else
#error Port me!
#endif

// Thread::Thread()
// Initialize thread state variables.
Thread::Thread()
:paused(false),stopped(false)
{
  
}//Thread::Thread

// Thread::~Thread()
// Destructor.
Thread::~Thread()
{
  
}//Thread::~Thread

// Thread::start()
// Call the os specific thread start routine.
// Pass instance a single parameter.
void Thread::start()
{
#ifdef WIN32
  ::_beginthread(startThread,0,(void*)this);
#endif

#ifdef POSIX
  ::pthread_create(&threadHandle,NULL, (void*(*)(void *))startThread,(void*)this);
#endif
}//Thread::start

// Thread::stop()
// Set the thread stop flag
void Thread::stop()
{
  stopped = true;
}//Thread::stop

// Thread::pause()
// Set the thread paused flag
void Thread::pause()
{
  paused = true;
}//Thread::pause

// Thread::resume()
// Unset the thread paused flag
void Thread::resume()
{
  paused = false;
}//Thread::resume

// Thread ::startThread(void* pThread)
// The actual thread function.  This calls
// exec on the passed instance.
void Thread::startThread(void* pThread)
{
  Thread* pT = (Thread*)pThread;
  pT->exec();
}//Thread::startThread

// Thread::handlePause()
// This is supposed to block
// until the resume function is called.
void Thread::handlePause()
{
  //not yet implemented.
}//Thread::handlePause()

// Thread::handleStop()
// Throw execption if stopped flag is set.
void Thread::handleStop()
{
  if(stopped){
    ThreadStoppedException e;
    throw e;
  }
}//Thread::handleStopped()
